@@ -14,6 +14,7 @@
 #include "ms-biff.h"
 #include "ms-excel-biff.h"
 #include "ms-container.h"
+#include <expr.h>
 
 typedef struct _ExcelSheet
 {
@@ -83,8 +84,9 @@ typedef struct _BiffFormatData {
 
 typedef struct _ExcelWorkbook
 {
-	MSContainer  container;
-	IOContext   *context;
+	MSContainer	  container;
+	IOContext	 *context;
+	WorkbookView	 *wbv;
 
 	GPtrArray	 *excel_sheets;
 	GHashTable	 *boundsheet_data_by_stream;
@@ -99,9 +101,11 @@ typedef struct _ExcelWorkbook
 	char		**global_strings;
 	guint32		  global_string_max;
 
-	gboolean warn_unsupported_graphs : 1;
+	gboolean          warn_unsupported_graphs;
 	GSList		 *delayed_names;
 
+	ExprTreeSharer   *expr_sharer;
+
 	/**
 	 * Gnumeric parallel workbook
    	 **/
@@ -111,7 +115,7 @@ typedef struct _ExcelWorkbook
 char       *biff_get_text (guint8 const *ptr, guint32 length, guint32 *byte_length);
 char const *biff_get_error_text (guint8 err);
 
-ExprTree	 	*ms_excel_workbook_get_name  (ExcelWorkbook const *ewb, int idx);
+GnmExpr const	 	*ms_excel_workbook_get_name  (ExcelWorkbook const *ewb, int idx);
 ExcelSheet		*ms_excel_workbook_get_sheet (ExcelWorkbook const *wb, guint idx);
 XLExternSheetV8 const	*ms_excel_workbook_get_externsheet_v8 (ExcelWorkbook const *wb,
 							       guint idx);
