@@ -25,15 +25,18 @@
 
 #include <gnumeric-config.h>
 #include <gnumeric.h>
-#include <expr.h>
 
 #include "boot.h"
 #include "ms-obj.h"
 #include "ms-chart.h"
 #include "ms-escher.h"
-#include "parse-util.h"
-#include "sheet-object-widget.h"
-#include "sheet-object-graphic.h"
+
+#include <expr.h>
+#include <parse-util.h>
+#include <sheet-object-widget.h>
+#include <sheet-object-graphic.h>
+
+#include <gsf/gsf-utils.h>
 
 #define GR_END                0x00
 #define GR_MACRO              0x04
@@ -92,7 +95,7 @@ ms_object_attr_new_ptr (MSObjAttrID id, gpointer val)
 }
 
 MSObjAttr *
-ms_object_attr_new_expr (MSObjAttrID id, ExprTree *expr)
+ms_object_attr_new_expr (MSObjAttrID id, GnmExpr const *expr)
 {
 	MSObjAttr *res = g_new (MSObjAttr, 1);
 
@@ -146,8 +149,8 @@ ms_object_attr_get_ptr  (MSObj *obj, MSObjAttrID id, gpointer default_value)
 	return attr->v.v_ptr;
 }
 
-ExprTree *
-ms_object_attr_get_expr (MSObj *obj, MSObjAttrID id, ExprTree *default_value)
+GnmExpr const *
+ms_object_attr_get_expr (MSObj *obj, MSObjAttrID id, GnmExpr const *default_value)
 {
 	MSObjAttr *attr;
 
@@ -169,7 +172,7 @@ ms_object_attr_destroy (MSObjAttr *attr)
 			g_free (attr->v.v_ptr);
 		} else if ((attr->id & MS_OBJ_ATTR_IS_EXPR_MASK) &&
 		    attr->v.v_expr != NULL) {
-			expr_tree_unref (attr->v.v_expr);
+			gnm_expr_unref (attr->v.v_expr);
 		}
 		g_free (attr);
 	}
@@ -233,7 +236,7 @@ ms_obj_delete (MSObj *obj)
 {
 	if (obj) {
 		if (obj->gnum_obj) {
-			gtk_object_unref (obj->gnum_obj);
+			g_object_unref (obj->gnum_obj);
 			obj->gnum_obj = NULL;
 		}
 		if (obj->attrs) {
@@ -263,19 +266,23 @@ ms_read_TXO (BiffQuery *q)
 		"At bottom", "Verticaly justified"
 	};
 
-	guint16 const options     = MS_OLE_GET_GUINT16 (q->data);
-	guint16 const orient      = MS_OLE_GET_GUINT16 (q->data + 2);
-	guint16 const text_len    = MS_OLE_GET_GUINT16 (q->data + 10);
-/*	guint16 const num_formats = MS_OLE_GET_GUINT16 (q->data + 12);*/
+	guint16 const options     = GSF_LE_GET_GUINT16 (q->data);
+	guint16 const orient      = GSF_LE_GET_GUINT16 (q->data + 2);
+	guint16 const text_len    = GSF_LE_GET_GUINT16 (q->data + 10);
+/*	guint16 const num_formats = GSF_LE_GET_GUINT16 (q->data + 12);*/
 	int const halign = (options >> 1) & 0x7;
 	int const valign = (options >> 4) & 0x7;
-	char         *text = g_new (char, text_len + 1);
+	char         *text;
 	guint16       peek_op;
 
+	if (text_len == 0)
+		return NULL;
+
 	g_return_val_if_fail (orient <= 3, NULL);
 	g_return_val_if_fail (1 <= halign && halign <= 4, NULL);
 	g_return_val_if_fail (1 <= valign && valign <= 4, NULL);
 
+	text = g_new (char, text_len + 1);
 	text [0] = '\0';
 	if (ms_biff_query_peek_next (q, &peek_op) &&
 	    peek_op == BIFF_CONTINUE) {
@@ -283,7 +290,7 @@ ms_read_TXO (BiffQuery *q)
 		int i, increment = 1;
 
 		ms_biff_query_next (q);
-		increment = (MS_OLE_GET_GUINT8 (q->data)) ? 2 : 1;
+		increment = (GSF_LE_GET_GUINT8 (q->data)) ? 2 : 1;
 		data = q->data + 1;
 
 		/*
@@ -302,7 +309,7 @@ ms_read_TXO (BiffQuery *q)
 		    peek_op == BIFF_CONTINUE)
 			ms_biff_query_next (q);
 		else
-			g_warning ("Unusual, TXO text with no formatting");
+			g_warning ("Unusual, TXO text with no formatting has 0x%x @ 0x%x", peek_op, q->streamPos);
 	} else if (text_len > 0)
 		g_warning ("TXO len of %d but no continue", text_len);
 
@@ -333,7 +340,7 @@ ms_obj_dump_impl (guint8 const *data, int len, int data_left, char const *name)
 		len = data_left - 4;
 	}
 	if (ms_excel_object_debug > 2)
-		ms_ole_dump (data, len+4);
+		gsf_mem_dump (data, len+4);
 	printf ("}; /* %s */\n", name);
 }
 #else
@@ -347,18 +354,16 @@ ms_obj_read_pre_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 	/* TODO : Lots of docs for these things.  Write the parser. */
 
 #if 0
-	guint32 const numObjects = MS_OLE_GET_GUINT16(q->data);
-	guint16 const flags = MS_OLE_GET_GUINT16(q->data+8);
+	guint32 const numObjects = GSF_LE_GET_GUINT16(q->data);
+	guint16 const flags = GSF_LE_GET_GUINT16(q->data+8);
 #endif
 	guint8 *anchor = g_malloc (MS_ANCHOR_SIZE);
 	memcpy (anchor, q->data+8, MS_ANCHOR_SIZE);
-	if (obj->attrs == NULL)
-		obj->attrs = ms_object_attr_bag_new ();
 	ms_object_attr_bag_insert (obj->attrs,
 		ms_object_attr_new_ptr (MS_OBJ_ATTR_ANCHOR, anchor));
 
-	obj->excel_type = MS_OLE_GET_GUINT16(q->data + 4);
-	obj->id         = MS_OLE_GET_GUINT32(q->data + 6);
+	obj->excel_type = GSF_LE_GET_GUINT16(q->data + 4);
+	obj->id         = GSF_LE_GET_GUINT32(q->data + 6);
 
 	return FALSE;
 }
@@ -384,14 +389,14 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 
 	/* Scan through the pseudo BIFF substream */
 	while (data_len_left > 0 && !hit_end) {
-		guint16 const record_type = MS_OLE_GET_GUINT16(data);
+		guint16 const record_type = GSF_LE_GET_GUINT16(data);
 
 		/* All the sub-records seem to have this layout
 		 * 2001/Mar/29 JEG : liars.  Ok not all records have this
 		 * layout.  Create a list box.  It seems to do something
 		 * unique.  It acts like an end, and has no length specified.
 		 */
-		guint16 len = MS_OLE_GET_GUINT16(data+2);
+		guint16 len = GSF_LE_GET_GUINT16(data+2);
 
 		/* 1st record must be COMMON_OBJ*/
 		g_return_val_if_fail (obj->excel_type >= 0 ||
@@ -426,7 +431,7 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 			guint16 pict_opt;
 			g_return_val_if_fail (len == 2, TRUE);
 
-			pict_opt = MS_OLE_GET_GUINT16(data+4);
+			pict_opt = GSF_LE_GET_GUINT16(data+4);
 
 #ifndef NO_DEBUG_EXCEL
 			if (ms_excel_object_debug >= 1) {
@@ -455,19 +460,19 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 		case GR_SCROLLBAR : {
 			ms_object_attr_bag_insert (obj->attrs,
 				ms_object_attr_new_uint (MS_OBJ_ATTR_SCROLLBAR_VALUE,
-					MS_OLE_GET_GUINT16 (data+8)));
+					GSF_LE_GET_GUINT16 (data+8)));
 			ms_object_attr_bag_insert (obj->attrs,
 				ms_object_attr_new_uint (MS_OBJ_ATTR_SCROLLBAR_MIN,
-					MS_OLE_GET_GUINT16 (data+10)));
+					GSF_LE_GET_GUINT16 (data+10)));
 			ms_object_attr_bag_insert (obj->attrs,
 				ms_object_attr_new_uint (MS_OBJ_ATTR_SCROLLBAR_MAX,
-					MS_OLE_GET_GUINT16 (data+12)));
+					GSF_LE_GET_GUINT16 (data+12)));
 			ms_object_attr_bag_insert (obj->attrs,
 				ms_object_attr_new_uint (MS_OBJ_ATTR_SCROLLBAR_INC,
-					MS_OLE_GET_GUINT16 (data+14)));
+					GSF_LE_GET_GUINT16 (data+14)));
 			ms_object_attr_bag_insert (obj->attrs,
 				ms_object_attr_new_uint (MS_OBJ_ATTR_SCROLLBAR_PAGE,
-					MS_OLE_GET_GUINT16 (data+16)));
+					GSF_LE_GET_GUINT16 (data+16)));
 			ms_obj_dump (data, len, data_len_left, "ScrollBar");
 			break;
 		}
@@ -477,8 +482,8 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 			break;
 
 		case GR_SCROLLBAR_FORMULA : {
-			guint16 const expr_len = MS_OLE_GET_GUINT16 (data+4);
-			ExprTree *ref = ms_container_parse_expr (container, data+10, expr_len);
+			guint16 const expr_len = GSF_LE_GET_GUINT16 (data+4);
+			GnmExpr const *ref = ms_container_parse_expr (container, data+10, expr_len);
 			if (ref != NULL)
 				ms_object_attr_bag_insert (obj->attrs,
 					ms_object_attr_new_expr (MS_OBJ_ATTR_SCROLLBAR_LINK, ref));
@@ -515,8 +520,8 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 		}
 
 		case GR_CHECKBOX_FORMULA : {
-			guint16 const expr_len = MS_OLE_GET_GUINT16 (data+4);
-			ExprTree *ref = ms_container_parse_expr (container, data+10, expr_len);
+			guint16 const expr_len = GSF_LE_GET_GUINT16 (data+4);
+			GnmExpr const *ref = ms_container_parse_expr (container, data+10, expr_len);
 			if (ref != NULL)
 				ms_object_attr_bag_insert (obj->attrs,
 					ms_object_attr_new_expr (MS_OBJ_ATTR_CHECKBOX_LINK, ref));
@@ -525,13 +530,19 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 		}
 
 		case GR_COMMON_OBJ_DATA : {
-			guint16 const options =MS_OLE_GET_GUINT16 (data+8);
+			guint16 const options =GSF_LE_GET_GUINT16 (data+8);
 
 			/* Multiple objects in 1 record ?? */
 			g_return_val_if_fail (obj->excel_type == -1, -1);
 
-			obj->excel_type = MS_OLE_GET_GUINT16(data+4);
-			obj->id = MS_OLE_GET_GUINT16(data+6);
+			obj->excel_type = GSF_LE_GET_GUINT16(data+4);
+			obj->id = GSF_LE_GET_GUINT16(data+6);
+
+			/* Undocumented.  It appears that combos for filters are marked
+			 * with flag 0x100
+			 */
+			obj->ignore_combo_in_filter =
+				(obj->excel_type == 0x14) && (options & 0x100);
 
 #ifndef NO_DEBUG_EXCEL
 			/* only print when debug is enabled */
@@ -551,7 +562,8 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 			if (ms_excel_object_debug > 4) {
 				/* According to the docs this should not fail
 				 * but there appears to be a flag at 0x200 for
-				 * scrollbars
+				 * scrollbars and 0x100 for combos
+				 * associated with filters.
 				 */
 				if ((options & 0x9fee) != 0)
 					printf ("WARNING : Why is option not 0 (%x)\n",
@@ -598,7 +610,7 @@ ms_obj_read_biff8_obj (BiffQuery *q, MSContainer *container, MSObj *obj)
 	/* The ftEnd record should have been the last */
 	if (data_len_left > 0) {
 		printf("OBJ : unexpected extra data after Object End record;\n");
-		ms_ole_dump (data, data_len_left);
+		gsf_mem_dump (data, data_len_left);
 		return TRUE;
 	}
 
@@ -667,7 +679,8 @@ ms_read_OBJ (BiffQuery *q, MSContainer *container, GHashTable *attrs)
 	obj->excel_type_name = NULL;
 	obj->id = -1;
 	obj->gnum_obj = NULL;
-	obj->attrs = attrs;
+	obj->attrs = (attrs != NULL) ? attrs : ms_object_attr_bag_new ();
+	obj->ignore_combo_in_filter = FALSE;
 
 #ifndef NO_DEBUG_EXCEL
 	if (ms_excel_object_debug > 0)
