@@ -561,663 +561,6 @@ gnumeric_bessely (FunctionEvalInfo *ei, Value **argv)
 	return value_new_float (yngnum (y, value_get_as_float (argv[0])));
 }
 
-/* Converts a complex number string into its coefficients.  Returns 0 if ok,
- * 1 if an error occurred.
- */
-static int
-value_get_as_complex (Value *val, complex_t *res, char *imunit)
-{
-	if (VALUE_IS_NUMBER (val)) {
-		complex_real (res, value_get_as_float (val));
-		*imunit = 'i';
-		return 0;
-	} else {
-		return complex_from_string (res,
-					    value_peek_string (val),
-					    imunit);
-	}
-}
-
-static Value *
-value_new_complex (const complex_t *c, char imunit)
-{
-	if (complex_real_p (c))
-		return value_new_float (c->re);
-	else {
-		char *s, f[5 + 4 * sizeof (int) + strlen (GNUM_FORMAT_g)];
-		Value *res;
-		sprintf (f, "%%.%d" GNUM_FORMAT_g, GNUM_DIG);
-		s = complex_to_string (c, f, f, imunit);
-		res = value_new_string (s);
-		g_free (s);
-		return res;
-	}
-}
-
-/***************************************************************************/
-
-static const char *help_complex = {
-	N_("@FUNCTION=COMPLEX\n"
-	   "@SYNTAX=COMPLEX(real,im[,suffix])\n"
-
-	   "@DESCRIPTION="
-	   "COMPLEX returns a complex number of the form x + yi. "
-	   "@real is the real and @im is the imaginary coefficient of "
-	   "the complex number.  @suffix is the suffix for the imaginary "
-	   "coefficient.  If it is omitted, COMPLEX uses 'i' by default."
-	   "\n"
-	   "If @suffix is neither 'i' nor 'j', COMPLEX returns #VALUE! "
-	   "error.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "COMPLEX(1,-1) equals 1-i.\n"
-	   "\n"
-	   "@SEEALSO=")
-};
-
-static Value *
-gnumeric_complex (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c;
-	const char *suffix;
-
-	complex_init (&c,
-		      value_get_as_float (argv[0]),
-		      value_get_as_float (argv[1]));
-	suffix = argv[2] ? value_peek_string (argv[2]) : "i";
-
-	if (strcmp (suffix, "i") != 0 && strcmp (suffix, "j") != 0)
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	return value_new_complex (&c, *suffix);
-}
-
-/***************************************************************************/
-
-static const char *help_imaginary = {
-	N_("@FUNCTION=IMAGINARY\n"
-	   "@SYNTAX=IMAGINARY(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMAGINARY returns the imaginary coefficient of a complex "
-	   "number.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMAGINARY(\"132-j\") equals -1.\n"
-	   "\n"
-	   "@SEEALSO=IMREAL")
-};
-
-static Value *
-gnumeric_imaginary (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c;
-	char imunit;
-
-	if (VALUE_IS_NUMBER (argv[0]))
-	        return value_new_float (0.0);
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	return value_new_float (c.im);
-}
-
-/***************************************************************************/
-
-static const char *help_imreal = {
-	N_("@FUNCTION=IMREAL\n"
-	   "@SYNTAX=IMREAL(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMREAL returns the real coefficient of a complex number.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "imreal(\"132-j\") equals 132.\n"
-	   "\n"
-	   "@SEEALSO=IMAGINARY")
-};
-
-
-static Value *
-gnumeric_imreal (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c;
-	char imunit;
-
-	if (VALUE_IS_NUMBER (argv[0]))
-		return value_duplicate (argv[0]);
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	return value_new_float (c.re);
-}
-
-/***************************************************************************/
-
-static const char *help_imabs = {
-	N_("@FUNCTION=IMABS\n"
-	   "@SYNTAX=IMABS(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMABS returns the absolute value of a complex number.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMABS(\"2-j\") equals 2.23606798.\n"
-	   "\n"
-	   "@SEEALSO=IMAGINARY,IMREAL")
-};
-
-
-static Value *
-gnumeric_imabs (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-	  return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	if (argv[0]->type != VALUE_STRING)
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	return value_new_float (complex_mod (&c));
-}
-
-/***************************************************************************/
-
-static const char *help_imconjugate = {
-	N_("@FUNCTION=IMCONJUGATE\n"
-	   "@SYNTAX=IMCONJUGATE(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMCONJUGATE returns the complex conjugate of a complex number.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMCONJUGATE(\"1-j\") equals 1+j.\n"
-	   "\n"
-	   "@SEEALSO=IMAGINARY,IMREAL")
-};
-
-static Value *
-gnumeric_imconjugate (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	if (argv[0]->type != VALUE_STRING)
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_conj (&res, &c);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imcos = {
-	N_("@FUNCTION=IMCOS\n"
-	   "@SYNTAX=IMCOS(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMCOS returns the cosine of a complex number.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMCOS(\"1+j\") equals 0.833730-0.988898j.\n"
-	   "\n"
-	   "@SEEALSO=IMSIN,IMTAN")
-};
-
-static Value *
-gnumeric_imcos (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_cos (&res, &c);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imtan = {
-	N_("@FUNCTION=IMTAN\n"
-	   "@SYNTAX=IMTAN(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMTAN returns the tangent of a complex number.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "\n"
-	   "@SEEALSO=IMSIN,IMCOS")
-};
-
-static Value *
-gnumeric_imtan (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_tan (&res, &c);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imexp = {
-	N_("@FUNCTION=IMEXP\n"
-	   "@SYNTAX=IMEXP(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMEXP returns the exponential of a complex number.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMEXP(\"2-j\") equals 3.992324-6.217676j.\n"
-	   "\n"
-	   "@SEEALSO=IMLN")
-};
-
-static Value *
-gnumeric_imexp (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_exp (&res, &c);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imargument = {
-	N_("@FUNCTION=IMARGUMENT\n"
-	   "@SYNTAX=IMARGUMENT(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMARGUMENT returns the argument theta of a complex number.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMARGUMENT(\"2-j\") equals -0.463647609.\n"
-	   "\n"
-	   "@SEEALSO=")
-};
-
-static Value *
-gnumeric_imargument (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	return value_new_float (complex_angle (&c));
-}
-
-/***************************************************************************/
-
-static const char *help_imln = {
-	N_("@FUNCTION=IMLN\n"
-	   "@SYNTAX=IMLN(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMLN returns the natural logarithm of a complex number. (The "
-	   "result will have an imaginary part between -pi and +pi.  The "
-	   "natural logarithm is not uniquely defined on complex numbers. "
-	   "You may need to add or subtract an even multiple of pi to the "
-	   "imaginary part.)\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMLN(\"3-j\") equals 1.15129-0.32175j.\n"
-	   "\n"
-	   "@SEEALSO=IMEXP,IMLOG2,IMLOG10")
-};
-
-static Value *
-gnumeric_imln (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_ln (&res, &c);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imlog2 = {
-	N_("@FUNCTION=IMLOG2\n"
-	   "@SYNTAX=IMLOG2(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMLOG2 returns the logarithm of a complex number in base 2.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMLOG2(\"3-j\") equals 1.66096-0.46419j.\n"
-	   "\n"
-	   "@SEEALSO=IMLN,IMLOG10")
-};
-
-static Value *
-gnumeric_imlog2 (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_ln (&res, &c);
-	complex_scale_real (&res, 1 / M_LN2gnum);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imlog10 = {
-	N_("@FUNCTION=IMLOG10\n"
-	   "@SYNTAX=IMLOG10(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMLOG10 returns the logarithm of a complex number in base 10.\n"
-	   "This function is Excel compatible. "
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMLOG10(\"3-j\") equals 0.5-0.13973j.\n"
-	   "\n"
-	   "@SEEALSO=IMLN,IMLOG2")
-};
-
-static Value *
-gnumeric_imlog10 (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_ln (&res, &c);
-	complex_scale_real (&res, 1 / M_LN10gnum);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_impower = {
-	N_("@FUNCTION=IMPOWER\n"
-	   "@SYNTAX=IMPOWER(inumber,number)\n"
-	   "@DESCRIPTION="
-	   "IMPOWER returns a complex number raised to a power.  @inumber is "
-	   "the complex number to be raised to a power and @number is the "
-	   "power to which you want to raise the complex number.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMPOWER(\"4-j\",2) equals 15-8j.\n"
-	   "\n"
-	   "@SEEALSO=IMSQRT")
-};
-
-static Value *
-gnumeric_impower (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t a, b, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &a, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	if (value_get_as_complex (argv[1], &b, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	if (complex_real_p (&a) && a.re <= 0 && !complex_real_p (&b))
-		return value_new_error (ei->pos, gnumeric_err_DIV0);
-
-	complex_pow (&res, &a, &b);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imdiv = {
-	N_("@FUNCTION=IMDIV\n"
-	   "@SYNTAX=IMDIV(inumber,inumber)\n"
-	   "@DESCRIPTION="
-	   "IMDIV returns the quotient of two complex numbers.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMDIV(\"2-j\",\"2+j\") equals 0.6-0.8j.\n"
-	   "\n"
-	   "@SEEALSO=IMPRODUCT")
-};
-
-static Value *
-gnumeric_imdiv (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t a, b, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &a, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	if (value_get_as_complex (argv[1], &b, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	if (complex_zero_p (&b))
-		return value_new_error (ei->pos, gnumeric_err_DIV0);
-
-	complex_div (&res, &a, &b);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imsin = {
-	N_("@FUNCTION=IMSIN\n"
-	   "@SYNTAX=IMSIN(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMSIN returns the sine of a complex number.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMSIN(\"1+j\") equals 1.29846+0.63496j.\n"
-	   "\n"
-	   "@SEEALSO=IMCOS,IMTAN")
-};
-
-static Value *
-gnumeric_imsin (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_sin (&res, &c);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imsqrt = {
-	N_("@FUNCTION=IMSQRT\n"
-	   "@SYNTAX=IMSQRT(inumber)\n"
-	   "@DESCRIPTION="
-	   "IMSQRT returns the square root of a complex number.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMSQRT(\"1+j\") equals 1.09868+0.4550899j.\n"
-	   "\n"
-	   "@SEEALSO=IMPOWER")
-};
-
-static Value *
-gnumeric_imsqrt (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t c, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_sqrt (&res, &c);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imsub = {
-	N_("@FUNCTION=IMSUB\n"
-	   "@SYNTAX=IMSUB(inumber,inumber)\n"
-	   "@DESCRIPTION="
-	   "IMSUB returns the difference of two complex numbers.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMSUB(\"3-j\",\"2+j\") equals 1-2j.\n"
-	   "\n"
-	   "@SEEALSO=IMSUM")
-};
-
-static Value *
-gnumeric_imsub (FunctionEvalInfo *ei, Value **argv)
-{
-	complex_t a, b, res;
-	char imunit;
-
-	if (value_get_as_complex (argv[0], &a, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	if (value_get_as_complex (argv[1], &b, &imunit))
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	complex_sub (&res, &a, &b);
-	return value_new_complex (&res, imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_improduct = {
-	N_("@FUNCTION=IMPRODUCT\n"
-	   "@SYNTAX=IMPRODUCT(inumber1[,inumber2,...])\n"
-	   "@DESCRIPTION="
-	   "IMPRODUCT returns the product of given complex numbers.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMPRODUCT(\"2-j\",\"4-2j\") equals 6-8j.\n"
-	   "\n"
-	   "@SEEALSO=IMDIV")
-};
-
-
-typedef enum {
-        Improduct, Imsum
-} eng_imoper_type_t;
-
-typedef struct {
-	complex_t         res;
-        char              imunit;
-        eng_imoper_type_t type;
-} eng_imoper_t;
-
-static Value *
-callback_function_imoper (const EvalPos *ep, Value *value, void *closure)
-{
-        eng_imoper_t *result = closure;
-	complex_t c;
-	char *imptr, dummy;
-
-	imptr = VALUE_IS_NUMBER (value) ? &dummy : &result->imunit;
-	if (value_get_as_complex (value, &c, imptr))
-		return value_new_error (ep, gnumeric_err_VALUE);
-
-	switch (result->type) {
-	case Improduct:
-		complex_mul (&result->res, &result->res, &c);
-	        break;
-	case Imsum:
-		complex_add (&result->res, &result->res, &c);
-	        break;
-	default:
-		abort ();
-	}
-
-        return NULL;
-}
-
-static Value *
-gnumeric_improduct (FunctionEvalInfo *ei, GnmExprList *expr_node_list)
-{
-	Value *v;
-        eng_imoper_t p;
-
-	p.type = Improduct;
-	p.imunit = 'j';
-	complex_real (&p.res, 1);
-
-        if ((v = function_iterate_argument_values (ei->pos,
-						   &callback_function_imoper,
-						   &p, expr_node_list,
-						   TRUE, TRUE)) != NULL)
-                return v;
-
-	return value_new_complex (&p.res, p.imunit);
-}
-
-/***************************************************************************/
-
-static const char *help_imsum = {
-	N_("@FUNCTION=IMSUM\n"
-	   "@SYNTAX=IMSUM(inumber,inumber)\n"
-	   "@DESCRIPTION="
-	   "IMSUM returns the sum of two complex numbers.\n"
-	   "This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "IMSUM(\"2-4j\",\"9-j\") equals 11-5j.\n"
-	   "\n"
-	   "@SEEALSO=IMSUB")
-};
-
-static Value *
-gnumeric_imsum (FunctionEvalInfo *ei, GnmExprList *expr_node_list)
-{
-	Value *v;
-        eng_imoper_t p;
-
-	p.type = Imsum;
-	p.imunit = 'j';
-	complex_real (&p.res, 0);
-
-        if ((v = function_iterate_argument_values (ei->pos,
-						   callback_function_imoper,
-						   &p, expr_node_list,
-						   TRUE, TRUE)) != NULL)
-                return v;
-
-	return value_new_complex (&p.res, p.imunit);
-}
-
 /***************************************************************************/
 
 static const char *help_convert = {
@@ -1808,45 +1151,47 @@ gnumeric_gestep (FunctionEvalInfo *ei, Value **argv)
 /***************************************************************************/
 
 const ModulePluginFunctionInfo engineering_functions[] = {
-        { "besseli",     "ff",   "xnum,ynum", &help_besseli, gnumeric_besseli, NULL, NULL, NULL },
-        { "besselk",     "ff",   "xnum,ynum", &help_besselk, gnumeric_besselk, NULL, NULL, NULL },
-        { "besselj",     "ff",   "xnum,ynum", &help_besselj, gnumeric_besselj, NULL, NULL, NULL },
-        { "bessely",     "ff",   "xnum,ynum", &help_bessely, gnumeric_bessely, NULL, NULL, NULL },
-        { "bin2dec",     "?",    "number", &help_bin2dec, gnumeric_bin2dec, NULL, NULL, NULL },
-        { "bin2hex",     "?|f",  "xnum,ynum", &help_bin2hex, gnumeric_bin2hex, NULL, NULL, NULL },
-        { "bin2oct",     "?|f",  "xnum,ynum", &help_bin2oct, gnumeric_bin2oct, NULL, NULL, NULL },
-        { "complex",     "ff|s", "real,im[,suffix]", &help_complex, gnumeric_complex, NULL, NULL, NULL },
-        { "convert",     "fss",  "number,from_unit,to_unit", &help_convert, gnumeric_convert, NULL, NULL, NULL },
-        { "dec2bin",     "?|f",  "xnum,ynum", &help_dec2bin, gnumeric_dec2bin, NULL, NULL, NULL },
-        { "dec2oct",     "?|f",  "xnum,ynum", &help_dec2oct, gnumeric_dec2oct, NULL, NULL, NULL },
-        { "dec2hex",     "?|f",  "xnum,ynum", &help_dec2hex, gnumeric_dec2hex, NULL, NULL, NULL },
-        { "delta",       "f|f",  "xnum,ynum", &help_delta, gnumeric_delta, NULL, NULL, NULL },
-        { "erf",         "f|f",  "lower,upper", &help_erf, gnumeric_erf , NULL, NULL, NULL },
-        { "erfc",        "f",    "number", &help_erfc, gnumeric_erfc, NULL, NULL, NULL },
-        { "gestep",      "f|f",  "xnum,ynum", &help_gestep, gnumeric_gestep, NULL, NULL, NULL },
-        { "hex2bin",     "?|f",  "xnum,ynum", &help_hex2bin, gnumeric_hex2bin, NULL, NULL, NULL },
-        { "hex2dec",     "?",    "number", &help_hex2dec, gnumeric_hex2dec, NULL, NULL, NULL },
-        { "hex2oct",     "?|f",  "xnum,ynum", &help_hex2oct, gnumeric_hex2oct, NULL, NULL, NULL },
-        { "imabs",       "?",    "inumber", &help_imabs, gnumeric_imabs, NULL, NULL, NULL },
-        { "imaginary",   "?",    "inumber", &help_imaginary, gnumeric_imaginary, NULL, NULL, NULL },
-        { "imargument",  "?",    "inumber", &help_imargument, gnumeric_imargument, NULL, NULL, NULL },
-        { "imconjugate", "?",    "inumber", &help_imconjugate, gnumeric_imconjugate, NULL, NULL, NULL },
-        { "imcos",       "?",    "inumber", &help_imcos, gnumeric_imcos, NULL, NULL, NULL },
-        { "imtan",       "?",    "inumber", &help_imtan, gnumeric_imtan, NULL, NULL, NULL },
-        { "imdiv",       "??",   "inumber,inumber", &help_imdiv, gnumeric_imdiv, NULL, NULL, NULL },
-        { "imexp",       "?",    "inumber", &help_imexp, gnumeric_imexp, NULL, NULL, NULL },
-        { "imln",        "?",    "inumber", &help_imln, gnumeric_imln, NULL, NULL, NULL },
-        { "imlog10",     "?",    "inumber", &help_imlog10, gnumeric_imlog10, NULL, NULL, NULL },
-        { "imlog2",      "?",    "inumber", &help_imlog2, gnumeric_imlog2, NULL, NULL, NULL },
-        { "impower",     "??",   "inumber,inumber", &help_impower, gnumeric_impower, NULL, NULL, NULL },
-        { "improduct",   NULL,   "inumber,inumber", &help_improduct, NULL, gnumeric_improduct, NULL, NULL },
-        { "imreal",      "?",    "inumber", &help_imreal, gnumeric_imreal, NULL, NULL, NULL },
-        { "imsin",       "?",    "inumber", &help_imsin, gnumeric_imsin, NULL, NULL, NULL },
-        { "imsqrt",      "?",    "inumber", &help_imsqrt, gnumeric_imsqrt, NULL, NULL, NULL },
-        { "imsub",       "??",   "inumber,inumber", &help_imsub, gnumeric_imsub, NULL, NULL, NULL },
-        { "imsum",       NULL,   "inumber,inumber", &help_imsum, NULL, gnumeric_imsum, NULL, NULL },
-        { "oct2bin",     "?|f",  "xnum,ynum", &help_oct2bin, gnumeric_oct2bin, NULL, NULL, NULL },
-        { "oct2dec",     "?",    "number", &help_oct2dec, gnumeric_oct2dec, NULL, NULL, NULL },
-        { "oct2hex",     "?|f",  "xnum,ynum", &help_oct2hex, gnumeric_oct2hex, NULL, NULL, NULL },
+        { "besseli",     "ff",   "xnum,ynum", &help_besseli,
+	  gnumeric_besseli, NULL, NULL, NULL },
+        { "besselk",     "ff",   "xnum,ynum", &help_besselk,
+	  gnumeric_besselk, NULL, NULL, NULL },
+        { "besselj",     "ff",   "xnum,ynum", &help_besselj,
+	  gnumeric_besselj, NULL, NULL, NULL },
+        { "bessely",     "ff",   "xnum,ynum", &help_bessely,
+	  gnumeric_bessely, NULL, NULL, NULL },
+        { "bin2dec",     "?",    "number", &help_bin2dec,
+	  gnumeric_bin2dec, NULL, NULL, NULL },
+        { "bin2hex",     "?|f",  "xnum,ynum", &help_bin2hex,
+	  gnumeric_bin2hex, NULL, NULL, NULL },
+        { "bin2oct",     "?|f",  "xnum,ynum", &help_bin2oct,
+	  gnumeric_bin2oct, NULL, NULL, NULL },
+        { "convert",     "fss",  "number,from_unit,to_unit", &help_convert,
+	  gnumeric_convert, NULL, NULL, NULL },
+        { "dec2bin",     "?|f",  "xnum,ynum", &help_dec2bin,
+	  gnumeric_dec2bin, NULL, NULL, NULL },
+        { "dec2oct",     "?|f",  "xnum,ynum", &help_dec2oct,
+	  gnumeric_dec2oct, NULL, NULL, NULL },
+        { "dec2hex",     "?|f",  "xnum,ynum", &help_dec2hex,
+	  gnumeric_dec2hex, NULL, NULL, NULL },
+        { "delta",       "f|f",  "xnum,ynum", &help_delta,
+	  gnumeric_delta, NULL, NULL, NULL },
+        { "erf",         "f|f",  "lower,upper", &help_erf,
+	  gnumeric_erf , NULL, NULL, NULL },
+        { "erfc",        "f",    "number", &help_erfc,
+	  gnumeric_erfc, NULL, NULL, NULL },
+        { "gestep",      "f|f",  "xnum,ynum", &help_gestep,
+	  gnumeric_gestep, NULL, NULL, NULL },
+        { "hex2bin",     "?|f",  "xnum,ynum", &help_hex2bin,
+	  gnumeric_hex2bin, NULL, NULL, NULL },
+        { "hex2dec",     "?",    "number", &help_hex2dec,
+	  gnumeric_hex2dec, NULL, NULL, NULL },
+        { "hex2oct",     "?|f",  "xnum,ynum", &help_hex2oct,
+	  gnumeric_hex2oct, NULL, NULL, NULL },
+        { "oct2bin",     "?|f",  "xnum,ynum", &help_oct2bin,
+	  gnumeric_oct2bin, NULL, NULL, NULL },
+        { "oct2dec",     "?",    "number", &help_oct2dec,
+	  gnumeric_oct2dec, NULL, NULL, NULL },
+        { "oct2hex",     "?|f",  "xnum,ynum", &help_oct2hex,
+	  gnumeric_oct2hex, NULL, NULL, NULL },
         {NULL}
 };
