@@ -7,10 +7,13 @@
 #include <gnumeric-config.h>
 #include <gnumeric.h>
 #include <Python.h>
+#ifdef WITH_PYGTK
+#include "pygobject.h"
+#endif
 #include <glib.h>
 #include <libgnome/libgnome.h>
 #include <gal/util/e-xml-utils.h>
-#include <gal/util/e-util.h>
+#include <gsf/gsf-impl-utils.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
@@ -18,6 +21,8 @@
 #include "sheet.h"
 #include "workbook-view.h"
 #include "value.h"
+#include "expr.h"
+#include "expr-impl.h"
 #include "io-context.h"
 #include "plugin-util.h"
 #include "plugin.h"
@@ -26,24 +31,22 @@
 #include "module-plugin-defs.h"
 #include "py-gnumeric.h"
 
-#define BROKEN_PY_INITIALIZE
-
 
 GNUMERIC_MODULE_PLUGIN_INFO_DECL;
 
-GtkType python_get_loader_type (ErrorInfo **ret_error);
+GType python_get_loader_type (ErrorInfo **ret_error);
 
 
 #define TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON            (gnumeric_plugin_loader_python_get_type ())
-#define GNUMERIC_PLUGIN_LOADER_PYTHON(obj)            (GTK_CHECK_CAST ((obj), TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON, GnumericPluginLoaderPython))
-#define GNUMERIC_PLUGIN_LOADER_PYTHON_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON, GnumericPluginLoaderPythonClass))
-#define IS_GNUMERIC_PLUGIN_LOADER_PYTHON(obj)         (GTK_CHECK_TYPE ((obj), TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON))
-#define IS_GNUMERIC_PLUGIN_LOADER_PYTHON_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON))
+#define GNUMERIC_PLUGIN_LOADER_PYTHON(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON, GnumericPluginLoaderPython))
+#define GNUMERIC_PLUGIN_LOADER_PYTHON_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON, GnumericPluginLoaderPythonClass))
+#define IS_GNUMERIC_PLUGIN_LOADER_PYTHON(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON))
+#define IS_GNUMERIC_PLUGIN_LOADER_PYTHON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GNUMERIC_PLUGIN_LOADER_PYTHON))
 
 typedef struct _GnumericPluginLoaderPython GnumericPluginLoaderPython;
 typedef struct _GnumericPluginLoaderPythonClass GnumericPluginLoaderPythonClass;
 
-GtkType gnumeric_plugin_loader_python_get_type (void);
+GType gnumeric_plugin_loader_python_get_type (void);
 
 #define PARENT_TYPE (gnumeric_plugin_loader_get_type ())
 
@@ -63,7 +66,7 @@ struct _GnumericPluginLoaderPythonClass {
 
 static GnumericPluginLoaderClass *parent_class = NULL;
 
-static void gnumeric_plugin_loader_python_set_attributes (GnumericPluginLoader *loader, GList *attr_names, GList *attr_values, ErrorInfo **ret_error);
+static void gnumeric_plugin_loader_python_set_attributes (GnumericPluginLoader *loader, GSList *attr_names, GSList *attr_values, ErrorInfo **ret_error);
 static void gnumeric_plugin_loader_python_load (GnumericPluginLoader *loader, ErrorInfo **ret_error);
 static void gnumeric_plugin_loader_python_unload (GnumericPluginLoader *loader, ErrorInfo **ret_error);
 static void gnumeric_plugin_loader_python_load_service_file_opener (GnumericPluginLoader *loader, PluginService *service, ErrorInfo **ret_error);
@@ -85,24 +88,28 @@ initialize_python_if_needed (void)
 #ifdef BROKEN_PY_INITIALIZE
 		int i;
 
-		/* Python's convertenviron has gotten into its head that it can
-		   write to the strings in the environment.  We have little choice
-		but to allocate a copy of everything. */
+		/* Before Python 2.0, Python's convertenviron would write to
+		   the strings in the environment.  We had little choice but
+		   to allocate a copy of everything. */
+
 		for (i = 0; environ[i]; i++)
 			environ[i] = g_strdup (environ[i]);
 #endif
 		Py_Initialize ();
 		PyEval_InitThreads ();
+#ifdef WITH_PYGTK
+		init_pygobject ();
+#endif
 		python_initialized = TRUE;
 	}
 }
 
 static void
-gnumeric_plugin_loader_python_set_attributes (GnumericPluginLoader *loader, GList *attr_names, GList *attr_values, ErrorInfo **ret_error)
+gnumeric_plugin_loader_python_set_attributes (GnumericPluginLoader *loader, GSList *attr_names, GSList *attr_values, ErrorInfo **ret_error)
 {
 	GnumericPluginLoaderPython *loader_python;
 	gchar *module_name = NULL;
-	GList *ln, *lv;
+	GSList *ln, *lv;
 
 	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_PYTHON (loader));
 	g_return_if_fail (ret_error != NULL);
@@ -133,7 +140,8 @@ static void
 gnumeric_plugin_loader_python_load (GnumericPluginLoader *loader, ErrorInfo **ret_error)
 {
 	GnumericPluginLoaderPython *loader_python;
-	gchar *python_file_extensions[] = {"py", "pyc", "pyo", NULL}, **file_ext;
+	const gchar *python_file_extensions[]
+		= {"py", "pyc", "pyo", NULL}, **file_ext;
 	InterpreterInfo *py_interpreter_info;
 	gchar *full_module_file_name = NULL;
 	FILE *f;
@@ -151,20 +159,20 @@ gnumeric_plugin_loader_python_load (GnumericPluginLoader *loader, ErrorInfo **re
 		clear_python_error_if_needed ();
 		*ret_error = error_info_new_str ("Cannot create new Python interpreter.");
 		return;
-	}	
+	}
 
 	for (file_ext = python_file_extensions; *file_ext != NULL; file_ext++) {
-		gchar *file_name;
-
-		file_name = g_strconcat (
-		            plugin_info_peek_dir_name (loader->plugin), PATH_SEP_STR,
-		            loader_python->module_name, ".", *file_ext, NULL);
-		if (!g_file_exists (file_name)) {
-			g_free (file_name);
-		} else {
-			full_module_file_name = file_name;
+		gchar *file_name = g_strconcat (
+			loader_python->module_name, ".", *file_ext, NULL);
+		gchar *path = g_build_filename (
+			plugin_info_peek_dir_name (loader->plugin),
+			file_name, NULL);
+		g_free (file_name);
+		if (g_file_test (path, G_FILE_TEST_EXISTS)) {
+			full_module_file_name = path;
 			break;
-		}
+		} else
+			g_free (path);
 	}
 	if (full_module_file_name == NULL) {
 		*ret_error = error_info_new_printf (
@@ -184,7 +192,8 @@ gnumeric_plugin_loader_python_load (GnumericPluginLoader *loader, ErrorInfo **re
 
 		modules = PyImport_GetModuleDict ();
 		g_return_if_fail (modules != NULL);
-		main_module = PyDict_GetItemString (modules, "__main__");
+		main_module = PyDict_GetItemString (modules,
+						    (char *) "__main__");
 		g_return_if_fail (main_module != NULL);
 		main_module_dict = PyModule_GetDict (main_module);
 		g_return_if_fail (main_module_dict != NULL);
@@ -214,26 +223,26 @@ gnumeric_plugin_loader_python_unload (GnumericPluginLoader *loader, ErrorInfo **
 
 static gint
 gnumeric_plugin_loader_python_info_get_extra_info_list (GnumericPluginLoader *loader,
-                                                        GList **ret_keys_list,
-                                                        GList **ret_values_list)
+                                                        GSList **ret_keys_list,
+                                                        GSList **ret_values_list)
 {
 	GnumericPluginLoaderPython *loader_python;
-	GList *keys_list = NULL, *values_list = NULL;
+	GSList *keys_list = NULL, *values_list = NULL;
 	gint n_items = 0;
 
 	g_return_val_if_fail (IS_GNUMERIC_PLUGIN_LOADER_PYTHON (loader), 0);
 	g_return_val_if_fail (ret_keys_list != NULL && ret_values_list != NULL, 0);
 
 	loader_python = GNUMERIC_PLUGIN_LOADER_PYTHON (loader);
-	keys_list = g_list_prepend (keys_list, g_strdup (_("Loader")));
-	values_list = g_list_prepend (values_list, g_strdup ("python"));
+	keys_list = g_slist_prepend (keys_list, g_strdup (_("Loader")));
+	values_list = g_slist_prepend (values_list, g_strdup ("python"));
 	n_items++;
-	keys_list = g_list_prepend (keys_list, g_strdup (_("Python module name")));
-	values_list = g_list_prepend (values_list, g_strdup (loader_python->module_name));
+	keys_list = g_slist_prepend (keys_list, g_strdup (_("Python module name")));
+	values_list = g_slist_prepend (values_list, g_strdup (loader_python->module_name));
 	n_items++;
 
-	*ret_keys_list = g_list_reverse (keys_list);
-	*ret_values_list = g_list_reverse (values_list);
+	*ret_keys_list = g_slist_reverse (keys_list);
+	*ret_values_list = g_slist_reverse (values_list);
 
 	return n_items;
 }
@@ -278,8 +287,9 @@ gnumeric_plugin_loader_python_class_init (GnumericPluginLoaderPythonClass *klass
 	gtk_object_class->destroy = gnumeric_plugin_loader_python_destroy;
 }
 
-E_MAKE_TYPE (gnumeric_plugin_loader_python, "GnumericPluginLoaderPython", GnumericPluginLoaderPython,
-	     &gnumeric_plugin_loader_python_class_init, gnumeric_plugin_loader_python_init, PARENT_TYPE)
+GSF_CLASS (GnumericPluginLoaderPython, gnumeric_plugin_loader_python,
+	   gnumeric_plugin_loader_python_class_init,
+	   gnumeric_plugin_loader_python_init, PARENT_TYPE)
 
 /*
  * Service - file_opener
@@ -291,19 +301,37 @@ typedef struct {
 } ServiceLoaderDataFileOpener;
 
 static gboolean
-gnumeric_plugin_loader_python_func_file_probe (GnumFileOpener const *fo, PluginService *service,
-                                               const gchar *file_name, FileProbeLevel pl)
+gnumeric_plugin_loader_python_func_file_probe (GnumFileOpener const *fo, 
+					       PluginService *service,
+                                               GsfInput *input, 
+					       FileProbeLevel pl)
 {
+#ifndef WITH_PYGTK
+	g_warning ("Probing from Python plugins requires gnome-python "
+		   "and Python bindings for libgsf");
+	return FALSE;
+#else	
 	ServiceLoaderDataFileOpener *loader_data;
-	PyObject *probe_result;
+	PyObject *probe_result = NULL;
+	PyObject *input_wrapper;
 	gboolean result;
 
 	g_return_val_if_fail (service != NULL, FALSE);
-	g_return_val_if_fail (file_name != NULL, FALSE);
+	g_return_val_if_fail (input != NULL, FALSE);
 
 	loader_data = (ServiceLoaderDataFileOpener *) plugin_service_get_loader_data (service);
 	switch_python_interpreter_if_needed (GNUMERIC_PLUGIN_LOADER_PYTHON (plugin_info_get_loader (service->plugin))->py_interpreter_info);
-	probe_result = PyObject_CallFunction (loader_data->python_func_file_probe, "s", file_name);
+	input_wrapper = pygobject_new (G_OBJECT (input));
+	if (input_wrapper == NULL) {
+		g_warning (convert_python_exception_to_string ());
+		clear_python_error_if_needed ();
+	}
+	if (input_wrapper != NULL) {
+		probe_result = PyObject_CallFunction
+			(loader_data->python_func_file_probe, 
+			 (char *) "O", input_wrapper);
+		Py_DECREF (input_wrapper);
+	}
 	if (probe_result != NULL) {
 		result = PyObject_IsTrue (probe_result);
 		Py_DECREF (probe_result);
@@ -313,25 +341,41 @@ gnumeric_plugin_loader_python_func_file_probe (GnumFileOpener const *fo, PluginS
 	}
 
 	return result;
+#endif
 }
 
 static void
-gnumeric_plugin_loader_python_func_file_open (GnumFileOpener const *fo, PluginService *service,
-                                              IOContext *io_context, WorkbookView *wb_view,
-                                              const gchar *file_name)
+gnumeric_plugin_loader_python_func_file_open (GnumFileOpener const *fo, 
+					      PluginService *service,
+                                              IOContext *io_context, 
+					      WorkbookView *wb_view,
+                                              GsfInput *input)
 {
+#ifndef WITH_PYGTK
+	gnumeric_io_error_string
+		(io_context,
+		 "File opening from python plugins requires gnome-python "
+		 "and Python bindings for libgsf");
+#else	
 	ServiceLoaderDataFileOpener *loader_data;
 	Sheet *sheet;
-	PyObject *open_result;
+	PyObject *open_result = NULL;
+	PyObject *input_wrapper;
 
 	g_return_if_fail (service != NULL);
-	g_return_if_fail (file_name != NULL);
+	g_return_if_fail (input != NULL);
 
 	loader_data = (ServiceLoaderDataFileOpener *) plugin_service_get_loader_data (service);
 	switch_python_interpreter_if_needed (GNUMERIC_PLUGIN_LOADER_PYTHON (plugin_info_get_loader (service->plugin))->py_interpreter_info);
 	sheet = sheet_new (wb_view_workbook (wb_view), _("Some name"));
-	open_result = PyObject_CallFunction (loader_data->python_func_file_open,
-	                                     "Ns", py_new_Sheet_object (sheet), file_name);
+	input_wrapper = pygobject_new (G_OBJECT (input));
+	if (input_wrapper != NULL) {
+		open_result = PyObject_CallFunction
+			(loader_data->python_func_file_open,
+			 (char *) "NO", 
+			 py_new_Sheet_object (sheet), input_wrapper);
+		Py_DECREF (input_wrapper);
+	}
 	if (open_result != NULL) {
 		Py_DECREF (open_result);
 		workbook_sheet_attach (wb_view_workbook (wb_view), sheet, NULL);
@@ -340,6 +384,7 @@ gnumeric_plugin_loader_python_func_file_open (GnumFileOpener const *fo, PluginSe
 		clear_python_error_if_needed ();
 		sheet_destroy (sheet);
 	}
+#endif
 }
 
 static void
@@ -432,8 +477,9 @@ gnumeric_plugin_loader_python_func_file_save (GnumFileSaver const *fs, PluginSer
 	saver_data = (ServiceLoaderDataFileSaver *) plugin_service_get_loader_data (service);
 	switch_python_interpreter_if_needed (GNUMERIC_PLUGIN_LOADER_PYTHON (plugin_info_get_loader (service->plugin))->py_interpreter_info);
 	py_workbook = py_new_Workbook_object (wb_view_workbook (wb_view));
-	save_result = PyObject_CallFunction (saver_data->python_func_file_save,
-	                                     "Ns", py_workbook, file_name);
+	save_result = PyObject_CallFunction
+		(saver_data->python_func_file_save,
+		 (char *) "Ns", py_workbook, file_name);
 	if (save_result != NULL) {
 		Py_DECREF (save_result);
 	} else {
@@ -502,19 +548,23 @@ call_python_function_args (FunctionEvalInfo *ei, Value **args)
 	ServiceLoaderDataFunctionGroup *loader_data;
 	PyObject *fn_info_tuple;
 	PyObject *python_fn;
+	FunctionDefinition const * fndef;
+
 	gint min_n_args, max_n_args, n_args;
 
 	g_return_val_if_fail (ei != NULL, NULL);
+	g_return_val_if_fail (ei->func_call != NULL, NULL);
 	g_return_val_if_fail (args != NULL, NULL);
 
-	service = (PluginService *) function_def_get_user_data (ei->func_def);
+	fndef = ei->func_call->func;
+	service = (PluginService *) function_def_get_user_data (fndef);
 	loader_data = (ServiceLoaderDataFunctionGroup *) plugin_service_get_loader_data (service);
 	switch_python_interpreter_if_needed (GNUMERIC_PLUGIN_LOADER_PYTHON (plugin_info_get_loader (service->plugin))->py_interpreter_info);
 	fn_info_tuple = PyDict_GetItemString (loader_data->python_fn_info_dict,
-	                                      (gchar *) function_def_get_name (ei->func_def));
+	                                      (gchar *) function_def_get_name (fndef));
 	g_assert (fn_info_tuple != NULL);
 	python_fn = PyTuple_GetItem (fn_info_tuple, 2);
-	function_def_count_args (ei->func_def, &min_n_args, &max_n_args);
+	function_def_count_args (fndef, &min_n_args, &max_n_args);
 	for (n_args = min_n_args; n_args < max_n_args && args[n_args] != NULL; n_args++) {
 		;
 	}
@@ -522,28 +572,31 @@ call_python_function_args (FunctionEvalInfo *ei, Value **args)
 }
 
 static Value *
-call_python_function_nodes (FunctionEvalInfo *ei, ExprList *expr_tree_list)
+call_python_function_nodes (FunctionEvalInfo *ei, GnmExprList *expr_tree_list)
 {
 	PluginService *service;
 	ServiceLoaderDataFunctionGroup *loader_data;
 	PyObject *python_fn;
+	FunctionDefinition const * fndef;
 	Value **values;
 	gint n_args, i;
-	ExprList *l;
+	GnmExprList *l;
 	Value *ret_value;
 
 	g_return_val_if_fail (ei != NULL, NULL);
+	g_return_val_if_fail (ei->func_call != NULL, NULL);
 
-	service = (PluginService *) function_def_get_user_data (ei->func_def);
+	fndef = ei->func_call->func;
+	service = (PluginService *) function_def_get_user_data (fndef);
 	loader_data = (ServiceLoaderDataFunctionGroup *) plugin_service_get_loader_data (service);
 	switch_python_interpreter_if_needed (GNUMERIC_PLUGIN_LOADER_PYTHON (plugin_info_get_loader (service->plugin))->py_interpreter_info);
 	python_fn = PyDict_GetItemString (loader_data->python_fn_info_dict,
-	                                  (gchar *) function_def_get_name (ei->func_def));
+	                                  (gchar *) function_def_get_name (fndef));
 
-	n_args = expr_list_length (expr_tree_list);
+	n_args = gnm_expr_list_length (expr_tree_list);
 	values = g_new (Value *, n_args);
 	for (i = 0, l = expr_tree_list; l != NULL; i++, l = l->next) {
-		values[i] = expr_eval (l->data, ei->pos, EVAL_PERMIT_NON_SCALAR);
+		values[i] = gnm_expr_eval (l->data, ei->pos, GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 	}
 	ret_value = call_python_function (python_fn, ei->pos, n_args, values);
 	for (i = 0; i < n_args; i++) {
@@ -554,7 +607,7 @@ call_python_function_nodes (FunctionEvalInfo *ei, ExprList *expr_tree_list)
 	return ret_value;
 }
 
-static gchar **
+static const gchar **
 python_function_get_gnumeric_help (PyObject *python_fn_info_dict, PyObject *python_fn,
                                    const gchar *fn_name)
 {
@@ -581,17 +634,19 @@ python_function_get_gnumeric_help (PyObject *python_fn_info_dict, PyObject *pyth
 	}
 	g_free (help_attr_name);
 
-	return (gchar **) PyCObject_AsVoidPtr (cobject_help_value);
+	return (const gchar **) PyCObject_AsVoidPtr (cobject_help_value);
 }
 
 static gboolean
 gnumeric_plugin_loader_python_func_get_full_function_info (PluginService *service,
-                                                           gchar const *fn_name,
-                                                           gchar **args_ptr,
-                                                           gchar **arg_names_ptr,
-                                                           gchar ***help_ptr,
-                                                           FunctionArgs **fn_args_ptr,
-                                                           FunctionNodes **fn_nodes_ptr)
+							   const gchar *fn_name,
+							   const gchar **args_ptr,
+							   const gchar **arg_names_ptr,
+							   const gchar ***help_ptr,
+							   FunctionArgs	 *fn_args_ptr,
+							   FunctionNodes *fn_nodes_ptr,
+							   FuncLinkHandle   *link,
+							   FuncUnlinkHandle *unlink)
 {
 	ServiceLoaderDataFunctionGroup *loader_data;
 	PyObject *fn_info_obj;
@@ -622,14 +677,16 @@ gnumeric_plugin_loader_python_func_get_full_function_info (PluginService *servic
 			                                               python_fn, fn_name);
 			*fn_args_ptr = &call_python_function_args;
 			*fn_nodes_ptr = NULL;
+			*link = NULL;
+			*unlink = NULL;
 			return TRUE;
 		} else {
 			clear_python_error_if_needed ();
 			return FALSE;
 		}
 	} else if (PyFunction_Check (fn_info_obj)) {
-		*args_ptr = "";
-		*arg_names_ptr = "";
+		*args_ptr = (char *) "";
+		*arg_names_ptr = (char *) "";
 		*help_ptr = python_function_get_gnumeric_help (loader_data->python_fn_info_dict,
 		                                               fn_info_obj, fn_name);
 		*fn_args_ptr = NULL;
@@ -666,7 +723,8 @@ gnumeric_plugin_loader_python_load_service_function_group (GnumericPluginLoader
 	if (python_fn_info_dict != NULL && PyDict_Check (python_fn_info_dict)) {
 		ServiceLoaderDataFunctionGroup *loader_data;
 
-		service_function_group->plugin_func_get_full_function_info = &gnumeric_plugin_loader_python_func_get_full_function_info;
+		service_function_group->plugin_func_get_full_function_info =
+			&gnumeric_plugin_loader_python_func_get_full_function_info;
 
 		loader_data = g_new (ServiceLoaderDataFunctionGroup, 1);
 		loader_data->python_fn_info_dict = (PyObject *) python_fn_info_dict;
@@ -710,7 +768,7 @@ gnumeric_plugin_loader_python_unload_service_function_group (GnumericPluginLoade
 
 /* ---------------------------------------- */
 
-GtkType
+GType
 python_get_loader_type (ErrorInfo **ret_error)
 {
 	*ret_error = NULL;
