@@ -2,15 +2,14 @@
 #define GNUMERIC_FUNC_H
 
 #include "gnumeric.h"
-#include "value.h"
-#include "expr.h"
-
-#include <libgnome/gnome-i18n.h>
+#include "dependent.h"
 
 /* Setup of the symbol table */
 void functions_init     (void);
+void functions_shutdown (void);
+
 /* Used to build manual */
-void function_dump_defs (const char *filename);
+void function_dump_defs (char const *filename);
 
 /******************************************************************************/
 /* Function category support */
@@ -25,7 +24,7 @@ struct _FunctionCategory {
 FunctionCategory *function_get_category     (gchar const *name);
 FunctionCategory *function_get_category_with_translation (gchar const *name,
                                                           gchar const *translation);
-FunctionCategory *function_category_get_nth (gint const n);
+FunctionCategory *function_category_get_nth (gint n);
 void function_category_add_func (FunctionCategory *, FunctionDefinition *);
 void function_category_remove_func (FunctionCategory *category, FunctionDefinition *fn_def);
 
@@ -62,16 +61,85 @@ void function_category_remove_func (FunctionCategory *category, FunctionDefiniti
  * "ff|ss" where the strings are optional
  **/
 
-void func_ref	(FunctionDefinition *fn_def);
-void func_unref (FunctionDefinition *fn_def);
+/* These are not supported yet */
+typedef enum {
+	FUNCTION_RETURNS_ARRAY = 0x01, /* eg transpose(), mmult() */
+	FUNCTION_RECALC_ALWAYS = 0x02, /* eg now(), today() */
+
+	/* For functions that are not exactly compatible with various import
+	 * formats.  We need to recalc their results to avoid changing values
+	 * unexpectedly when we recalc later.  This probably needs to be done
+	 * on a per import format basis.  It may not belong here.
+	 */
+	FUNCTION_RECALC_ONLOAD = 0x04
+
+	/* TODO : Are there other forms or recalc we need to think about ? */
+} FunctionFlags;
+
+typedef enum { FUNCTION_ARGS, FUNCTION_NODES, FUNCTION_NAMEONLY } FuncType;
+
+typedef Value *(*FunctionArgs)  (FunctionEvalInfo *ei, Value **args);
+typedef Value *(*FunctionNodes) (FunctionEvalInfo *ei, GnmExprList *nodes);
+
+struct _FunctionEvalInfo {
+	EvalPos const *pos;
+	GnmExprFunction const *func_call;
+};
+
+typedef DependentFlags	(*FuncLinkHandle) 	(FunctionEvalInfo *ei);
+typedef void		(*FuncUnlinkHandle) 	(FunctionEvalInfo *ei);
+typedef gboolean (*FunctionGetFullInfoCallback) (FunctionDefinition *fn_def,
+                                                 gchar const **args_ptr,
+                                                 gchar const **arg_names_ptr,
+                                                 gchar const ***help_ptr,
+                                                 FunctionArgs	  *fn_args_ptr,
+                                                 FunctionNodes	  *fn_nodes_ptr,
+						 FuncLinkHandle	  *link,
+						 FuncUnlinkHandle *unlink);
+
+typedef enum {
+    FUNC_IMPL_STATUS_UNIMPLEMENTED,
+    FUNC_IMPL_STATUS_SUBSET,
+    FUNC_IMPL_STATUS_COMPLETE,
+    FUNC_IMPL_STATUS_SUPERSET,
+    FUNC_IMPL_STATUS_SUBSET_WITH_EXTENSIONS,
+    FUNC_IMPL_STATUS_CLAIMED
+} FunctionImplStatus;
+typedef enum {
+    FUNC_TEST_STATUS_UNTESTED,
+    FUNC_TEST_STATUS_BASIC,
+    FUNC_TEST_STATUS_EXHAUSTIVE,
+    FUNC_TEST_STATUS_CLAIMED
+} FunctionTestStatus;
+
+struct _FunctionDefinition {
+	FunctionGetFullInfoCallback get_full_info_callback;
+	FunctionFlags flags;
+	gchar   const *name;
+	gchar   const *named_arguments;
+	gchar   const **help;
+	FuncType       fn_type;
+	union {
+		FunctionNodes fn_nodes;
+		struct {
+			char const *arg_spec;
+			FunctionArgs  func;
+			int min_args, max_args;
+			char *arg_types;
+		} args;
+	} fn;
+	FuncLinkHandle   link;
+	FuncUnlinkHandle unlink;
+	gpointer     user_data;
+	gint         ref_count;
+	FunctionImplStatus	impl_status;
+	FunctionTestStatus	test_status;
+};
+void func_ref	 (FunctionDefinition *fn_def);
+void func_unref  (FunctionDefinition *fn_def);
 gint func_get_ref_count (FunctionDefinition *fn_def);
 
-typedef gboolean (*FunctionGetFullInfoCallback) (FunctionDefinition *fn_def,
-                                                 gchar **args_ptr,
-                                                 gchar **arg_names_ptr,
-                                                 gchar ***help_ptr,
-                                                 FunctionArgs **fn_args_ptr,
-                                                 FunctionNodes **fn_nodes_ptr);
+void func_def_load (FunctionDefinition *fn_def);
 
 FunctionDefinition *func_lookup_by_name	(gchar const *fn_name,
                                          Workbook const *optional_scope);
@@ -81,14 +149,14 @@ FunctionDefinition *function_add_args	(FunctionCategory *category,
                                          gchar const *name,
                                          gchar const *args,
                                          gchar const *arg_names,
-                                         gchar **help,
-                                         FunctionArgs *fn);
+                                         gchar const **help,
+                                         FunctionArgs fn);
 FunctionDefinition *function_add_nodes	(FunctionCategory *category,
                                          gchar const *name,
                                          gchar const *args,
                                          gchar const *arg_names,
-                                         gchar **help,
-                                         FunctionNodes *fn);
+                                         gchar const **help,
+                                         FunctionNodes fn);
 FunctionDefinition *function_add_name_only (FunctionCategory *category,
                                             gchar const *name,
                                             FunctionGetFullInfoCallback callback);
@@ -99,33 +167,42 @@ gpointer function_def_get_user_data    (FunctionDefinition const *fn_def);
 void     function_def_set_user_data    (FunctionDefinition *fn_def,
                                         gpointer user_data);
 
-const char *function_def_get_name      (FunctionDefinition const *fn_def);
+char const *function_def_get_name      (FunctionDefinition const *fn_def);
 void        function_def_count_args    (FunctionDefinition const *fn_def,
                                         gint *min, int *max);
 char        function_def_get_arg_type  (FunctionDefinition const *fn_def,
                                         gint arg_idx);
+char const *function_def_get_arg_type_string  (FunctionDefinition const *fn_def,
+                                        gint arg_idx);
+char       *function_def_get_arg_name  (FunctionDefinition const *fn_def,
+                                        gint arg_idx);
 
-Value *function_call_with_list	     (FunctionEvalInfo *ei, ExprList *args);
-Value *function_call_with_values     (const EvalPos *ep, const gchar *name,
+void function_set_link_handlers (FunctionDefinition *fn_def,
+				 FuncLinkHandle   link,
+				 FuncUnlinkHandle unlink);
+
+Value *function_call_with_list	     (FunctionEvalInfo *ei, GnmExprList *args,
+				      GnmExprEvalFlags flags);
+Value *function_call_with_values     (EvalPos const *ep, gchar const *name,
                                       gint argc, Value *values []);
 Value *function_def_call_with_values (EvalPos const *ep, FunctionDefinition const *fn,
                                       gint argc, Value *values []);
 
 /* Utilies to interate through ranges and argument lists */
-typedef Value * (*FunctionIterateCB) (const EvalPos *ep,
+typedef Value * (*FunctionIterateCB) (EvalPos const *ep,
                                       Value *value, gpointer user_data);
-Value *function_iterate_argument_values	(const EvalPos     *ep,
+Value *function_iterate_argument_values	(EvalPos const	   *ep,
                                          FunctionIterateCB  cb,
                                          gpointer           user_data,
-                                         ExprList          *expr_node_list,
+                                         GnmExprList       *expr_node_list,
                                          gboolean           strict,
-                                         gboolean           ignore_blank);
-Value *function_iterate_do_value	(const EvalPos      *ep,
-                                     FunctionIterateCB   cb,
-                                     gpointer            user_data,
-                                     Value              *value,
-                                     gboolean            strict,
-                                     gboolean            ignore_blank);
+                                         CellIterFlags	    iter_flags);
+Value *function_iterate_do_value	(EvalPos const      *ep,
+					 FunctionIterateCB   cb,
+					 gpointer            user_data,
+					 Value              *value,
+					 gboolean            strict,
+					 CellIterFlags	     iter_flags);
 
 /******************************************************************************/
 
@@ -138,7 +215,7 @@ typedef struct {
 } TokenizedHelp;
 
 TokenizedHelp *tokenized_help_new     (FunctionDefinition const *fn_def);
-const gchar   *tokenized_help_find    (TokenizedHelp *tok, const gchar *token);
+gchar   const *tokenized_help_find    (TokenizedHelp *tok, gchar const *token);
 void           tokenized_help_destroy (TokenizedHelp *tok);
 
 #endif /* GNUMERIC_FUNC_H */
