@@ -16,13 +16,16 @@ typedef struct _WorkbookView		WorkbookView;
 typedef struct _WorkbookControl		WorkbookControl;
 
 typedef struct _Sheet			Sheet;
+typedef struct _SheetView		SheetView;
 typedef struct _SheetControl		SheetControl;
 
 typedef struct _SolverParameters	SolverParameters;
 
 typedef struct _SheetObject		SheetObject;
 typedef struct _SheetObjectAnchor	SheetObjectAnchor;
+typedef struct _GnmHLink		GnmHLink;
 
+typedef struct _GnmDepContainer		GnmDepContainer;
 typedef struct _Dependent		Dependent;
 typedef struct _Cell			Cell;
 typedef struct _CellComment		CellComment;
@@ -82,11 +85,12 @@ typedef struct _ParsePos	        ParsePos;
 typedef struct _ParseError	        ParseError;
 typedef struct _FunctionEvalInfo        FunctionEvalInfo;
 typedef struct _FunctionDefinition      FunctionDefinition;
+typedef struct _ErrorInfo		ErrorInfo;
 
 typedef struct _PrintInformation        PrintInformation;
 typedef struct _String	 	        String;
 
-typedef struct _GnmDepContainer		GnmDepContainer;
+typedef struct _GnumFileOpener		GnumFileOpener;	/* TODO rename to GnmFileFormatReader */
 
 typedef struct _StyleFormat	        StyleFormat;
 typedef struct _StyleFont	        StyleFont;
@@ -115,8 +119,13 @@ struct _GlobalRange {
 	Range  range;
 };
 
-typedef Value * (*ForeachCellCB)(Sheet *sheet, int col, int row,
-				 Cell *cell, void *user_data);
+typedef enum {
+	CELL_ITER_ALL 		= 0,
+	CELL_ITER_IGNORE_BLANK  = 1 << 0,
+	CELL_ITER_IGNORE_HIDDEN = 1 << 1
+} CellIterFlags;
+typedef Value *(*CellIterFunc) (Sheet *sheet, int col, int row,
+				Cell *cell, gpointer user_data);
 
 typedef enum _SpanCalcFlags {
 	SPANCALC_SIMPLE 	= 0x0,	/* Just calc spans */
@@ -126,6 +135,13 @@ typedef enum _SpanCalcFlags {
 	SPANCALC_NO_DRAW	= 0x8	/* Do not queue a redraw */
 } SpanCalcFlags;
 
+typedef enum
+{
+	GNM_EXPR_EVAL_STRICT		= 0x0,
+	GNM_EXPR_EVAL_PERMIT_NON_SCALAR	= 0x1,
+	GNM_EXPR_EVAL_PERMIT_EMPTY	= 0x2
+} GnmExprEvalFlags;
+
 typedef struct _SearchReplace           SearchReplace;
 
 typedef struct _gnm_mem_chunk		gnm_mem_chunk;
