@@ -3,6 +3,7 @@
  *
  * Author:
  *   Jon Kre Hellan <hellan@acm.org>
+ *   Morten Welinder <terra@diku.dk>
  *
  * Copyright (C) 2002, Jon Kre Hellan
  */
@@ -16,6 +17,7 @@
 #include "workbook-view.h"
 #include "plugin.h"
 #include "workbook.h"
+#include "gui-file.h"
 #include "gnumeric-gconf.h"
 
 #include <libgnome/gnome-i18n.h>
@@ -25,9 +27,14 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <time.h>
+#include <string.h>
 
-static	int gnumeric_show_version = FALSE;
-static	char *dump_file_name = NULL;
+#ifdef WITH_BONOBO
+#include <bonobo/bonobo-main.h>
+#endif
+
+static int gnumeric_show_version = FALSE;
+static char *dump_file_name = NULL;
 
 char const *gnumeric_lib_dir = GNUMERIC_LIBDIR;
 char const *gnumeric_data_dir = GNUMERIC_DATADIR;
@@ -82,6 +89,7 @@ warn_about_ancient_gnumerics (const char *binary, WorkbookControl *wbc)
 	int days = 180;
 
 	if (binary &&
+	    strchr (binary, '/') != NULL &&
 	    stat (binary, &buf) != -1 &&
 	    buf.st_mtime != -1 &&
 	    now - buf.st_mtime > days * 24 * 60 * 60) {
@@ -124,7 +132,7 @@ main (int argc, char *argv [])
 	gnm_common_init ();
 
 	if (dump_file_name)
-		return gnm_dump_func_defs (dump_file_name); 
+		return gnm_dump_func_defs (dump_file_name);
 
 	/* Load selected files */
 	if (ctx)
@@ -147,7 +155,8 @@ main (int argc, char *argv [])
 	if (startup_files) {
 		int i;
 		for (i = 0; startup_files [i]  && !initial_workbook_open_complete ; i++) {
- 			if (wb_view_open (startup_files[i], wbc, TRUE, NULL))
+ 			if (gui_file_read (WORKBOOK_CONTROL_GUI (wbc),
+					   startup_files[i], NULL))
   				opened_workbook = TRUE;
 
 			/* cheesy attempt to keep the ui from freezing during load */
