@@ -15,7 +15,7 @@
 
 #include <libgnome/libgnome.h>
 #include <gal/util/e-xml-utils.h>
-#include <gal/util/e-util.h>
+#include <gsf/gsf-impl-utils.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
@@ -39,7 +39,7 @@ struct _GnumericPluginLoaderModuleClass {
 
 static GnumericPluginLoaderClass *parent_class = NULL;
 
-static void gnumeric_plugin_loader_module_set_attributes (GnumericPluginLoader *loader, GList *attr_names, GList *attr_values, ErrorInfo **ret_error);
+static void gnumeric_plugin_loader_module_set_attributes (GnumericPluginLoader *loader, GSList *attr_names, GSList *attr_values, ErrorInfo **ret_error);
 static void gnumeric_plugin_loader_module_load (GnumericPluginLoader *loader, ErrorInfo **ret_error);
 static void gnumeric_plugin_loader_module_unload (GnumericPluginLoader *loader, ErrorInfo **ret_error);
 
@@ -53,12 +53,12 @@ static void gnumeric_plugin_loader_module_load_service_plugin_loader (GnumericPl
 
 static void
 gnumeric_plugin_loader_module_set_attributes (GnumericPluginLoader *loader,
-                                              GList *attr_names, GList *attr_values,
+                                              GSList *attr_names, GSList *attr_values,
                                               ErrorInfo **ret_error)
 {
 	GnumericPluginLoaderModule *loader_module;
 	gchar *module_file_name = NULL;
-	GList *ln, *lv;
+	GSList *ln, *lv;
 
 	g_return_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader));
 	g_return_if_fail (ret_error != NULL);
@@ -101,8 +101,8 @@ gnumeric_plugin_loader_module_load (GnumericPluginLoader *loader, ErrorInfo **re
 		ModulePluginFileStruct *plugin_file_struct = NULL;
 		gpointer plugin_init_func = NULL, plugin_cleanup_func = NULL;
 
-		full_module_file_name = g_concat_dir_and_file (plugin_info_peek_dir_name (loader->plugin),
-		                                               loader_module->module_file_name);
+		full_module_file_name = g_build_filename (plugin_info_peek_dir_name (loader->plugin),
+							  loader_module->module_file_name, NULL);
 		handle = g_module_open (full_module_file_name, 0);
 		if (handle != NULL) {
 			g_module_symbol (handle, "plugin_file_struct", (gpointer) &plugin_file_struct);
@@ -179,26 +179,26 @@ gnumeric_plugin_loader_module_unload (GnumericPluginLoader *loader, ErrorInfo **
 
 static gint
 gnumeric_plugin_loader_module_info_get_extra_info_list (GnumericPluginLoader *loader,
-                                                        GList **ret_keys_list,
-                                                        GList **ret_values_list)
+                                                        GSList **ret_keys_list,
+                                                        GSList **ret_values_list)
 {
 	GnumericPluginLoaderModule *loader_module;
-	GList *keys_list = NULL, *values_list = NULL;
+	GSList *keys_list = NULL, *values_list = NULL;
 	gint n_items = 0;
 
 	g_return_val_if_fail (IS_GNUMERIC_PLUGIN_LOADER_MODULE (loader), 0);
 	g_return_val_if_fail (ret_keys_list != NULL && ret_values_list != NULL, 0);
 
 	loader_module = GNUMERIC_PLUGIN_LOADER_MODULE (loader);
-	keys_list = g_list_prepend (keys_list, g_strdup (_("Loader")));
-	values_list = g_list_prepend (values_list, g_strdup ("g_module"));
+	keys_list = g_slist_prepend (keys_list, g_strdup (_("Loader")));
+	values_list = g_slist_prepend (values_list, g_strdup ("g_module"));
 	n_items++;
-	keys_list = g_list_prepend (keys_list, g_strdup (_("Module file name")));
-	values_list = g_list_prepend (values_list, g_strdup (loader_module->module_file_name));
+	keys_list = g_slist_prepend (keys_list, g_strdup (_("Module file name")));
+	values_list = g_slist_prepend (values_list, g_strdup (loader_module->module_file_name));
 	n_items++;
 
-	*ret_keys_list = g_list_reverse (keys_list);
-	*ret_values_list = g_list_reverse (values_list);
+	*ret_keys_list = g_slist_reverse (keys_list);
+	*ret_values_list = g_slist_reverse (values_list);
 
 	return n_items;
 }
@@ -245,7 +245,9 @@ gnumeric_plugin_loader_module_class_init (GnumericPluginLoaderModuleClass *klass
 	gtk_object_class->destroy = gnumeric_plugin_loader_module_destroy;
 }
 
-E_MAKE_TYPE (gnumeric_plugin_loader_module, "GnumericPluginLoaderModule", GnumericPluginLoaderModule, &gnumeric_plugin_loader_module_class_init, gnumeric_plugin_loader_module_init, PARENT_TYPE)
+GSF_CLASS (GnumericPluginLoaderModule, gnumeric_plugin_loader_module,
+	   gnumeric_plugin_loader_module_class_init,
+	   gnumeric_plugin_loader_module_init, PARENT_TYPE)
 
 /*
  * Service - general
@@ -356,37 +358,37 @@ gnumeric_plugin_loader_module_load_service_general (GnumericPluginLoader *loader
  */
 
 typedef struct {
-	gboolean (*module_func_file_probe) (GnumFileOpener const *fo, const gchar *file_name,
+	gboolean (*module_func_file_probe) (GnumFileOpener const *fo, GsfInput *input,
 	                                    FileProbeLevel pl);
 	void (*module_func_file_open) (GnumFileOpener const *fo, IOContext *io_context,
-	                               WorkbookView *wbv, const gchar *filename);
+	                               WorkbookView *wbv, GsfInput *input);
 } ServiceLoaderDataFileOpener;
 
 static gboolean
 gnumeric_plugin_loader_module_func_file_probe (GnumFileOpener const *fo, PluginService *service,
-                                               const gchar *file_name, FileProbeLevel pl)
+                                               GsfInput *input, FileProbeLevel pl)
 {
 	ServiceLoaderDataFileOpener *loader_data;
 
 	g_return_val_if_fail (service != NULL, FALSE);
-	g_return_val_if_fail (file_name != NULL, FALSE);
+	g_return_val_if_fail (input != NULL, FALSE);
 
 	loader_data = (ServiceLoaderDataFileOpener *) plugin_service_get_loader_data (service);
-	return loader_data->module_func_file_probe (fo, file_name, pl);
+	return loader_data->module_func_file_probe (fo, input, pl);
 }
 
 static void
 gnumeric_plugin_loader_module_func_file_open (GnumFileOpener const *fo, PluginService *service,
                                               IOContext *io_context, WorkbookView *wbv,
-                                              const gchar *file_name)
+                                              GsfInput *input)
 {
 	ServiceLoaderDataFileOpener *loader_data;
 
 	g_return_if_fail (service != NULL);
-	g_return_if_fail (file_name != NULL);
+	g_return_if_fail (input != NULL);
 
 	loader_data = (ServiceLoaderDataFileOpener *) plugin_service_get_loader_data (service);
-	loader_data->module_func_file_open (fo, io_context, wbv, file_name);
+	loader_data->module_func_file_open (fo, io_context, wbv, input);
 }
 
 static void
@@ -526,12 +528,14 @@ typedef struct {
 
 static gboolean
 gnumeric_plugin_loader_module_func_get_full_function_info (PluginService *service,
-                                                           gchar const *fn_name,
-                                                           gchar **args_ptr,
-                                                           gchar **arg_names_ptr,
-                                                           gchar ***help_ptr,
-                                                           FunctionArgs **fn_args_ptr,
-                                                           FunctionNodes **fn_nodes_ptr)
+                                                           const gchar *fn_name,
+                                                           const gchar **args_ptr,
+                                                           const gchar **arg_names_ptr,
+                                                           const gchar ***help_ptr,
+                                                           FunctionArgs	    *fn_args_ptr,
+                                                           FunctionNodes    *fn_nodes_ptr,
+							   FuncLinkHandle   *fn_link,
+							   FuncUnlinkHandle *fn_unlink)
 {
 	ServiceLoaderDataFunctionGroup *loader_data;
 	gpointer func_index_ptr;
@@ -550,6 +554,8 @@ gnumeric_plugin_loader_module_func_get_full_function_info (PluginService *servic
 		*help_ptr = fn_info->help;
 		*fn_args_ptr = fn_info->fn_args;
 		*fn_nodes_ptr = fn_info->fn_nodes;
+		*fn_link   = fn_info->link;
+		*fn_unlink = fn_info->unlink;
 		return TRUE;
 	} else {
 		return FALSE;
@@ -626,16 +632,16 @@ gnumeric_plugin_loader_module_unload_service_function_group (GnumericPluginLoade
  */
 
 typedef struct {
-	GtkType (*module_func_get_loader_type) (ErrorInfo **ret_error);
+	GType (*module_func_get_loader_type) (ErrorInfo **ret_error);
 } ServiceLoaderDataPluginLoader;
 
-static GtkType
+static GType
 gnumeric_plugin_loader_module_func_get_loader_type (PluginService *service,
                                                     ErrorInfo **ret_error)
 {
 	ServiceLoaderDataPluginLoader *loader_data;
 	ErrorInfo *error;
-	GtkType loader_type;
+	GType loader_type;
 
 	g_return_val_if_fail (service != NULL, 0);
 	g_return_val_if_fail (ret_error != NULL, 0);
@@ -647,7 +653,7 @@ gnumeric_plugin_loader_module_func_get_loader_type (PluginService *service,
 		return loader_type;
 	} else {
 		*ret_error = error;
-		return (GtkType) 0;
+		return (GType) 0;
 	}
 }
 
