@@ -26,7 +26,7 @@
 #include <commands.h>
 #include <gnm-marshalers.h>
 
-#include <gal/util/e-util.h>
+#include <gsf/gsf-impl-utils.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkcelleditable.h>
 #include <ctype.h>
@@ -410,13 +410,13 @@ gnm_expr_entry_rangesel_start (GnumericExprEntry *gee)
 		rs->text_start = 0;
 		rs->text_end = last;
 	} else {
-		for (start = cursor; start >= 0; start = g_utf8_prev_char (text + start) - text) {
+		for (start = cursor; start > 0; start = g_utf8_prev_char (text + start) - text) {
 			gunichar c = g_utf8_get_char (text + start);
 			if (!isalnum (c)) {
+				start = g_utf8_next_char (text + start) - text;
 				break;
 			}
 		}
-		start = g_utf8_next_char (text + start) - text;
 		rs->text_start = (cursor < start) ? cursor : start;
 		for (end = cursor; end < last; end = g_utf8_next_char (text + end) - text) {
 			gunichar c = g_utf8_get_char (text + end);
@@ -540,9 +540,9 @@ cb_gee_button_press_event (GtkEntry *entry, GdkEventButton *event, GnumericExprE
 
 
 static gboolean
-cb_gee_key_press_event (GtkEntry *entry,
-		     GdkEventKey *event,
-		     GnumericExprEntry *gee)
+cb_gee_key_press_event (GtkEntry	  *entry,
+			GdkEventKey	  *event,
+			GnumericExprEntry *gee)
 {
 	WorkbookControlGUI *wbcg  = gee->wbcg;
 	int state = gnumeric_filter_modifiers (event->state);
@@ -619,16 +619,14 @@ cb_gee_key_press_event (GtkEntry *entry,
 
 		if (state == GDK_CONTROL_MASK ||
 		    state == (GDK_CONTROL_MASK|GDK_SHIFT_MASK)) {
-			ParsePos pos;
 			gboolean const is_array = (state & GDK_SHIFT_MASK);
 			char const *text = gtk_entry_get_text (
 				wbcg_get_entry (wbcg));
-			Sheet *sheet = wbcg->editing_sheet;
 
 			/* Be careful to use the editing sheet */
 			cmd_area_set_text (WORKBOOK_CONTROL (wbcg),
-				parse_pos_init (&pos, NULL, sheet,
-					sheet->edit_pos.col, sheet->edit_pos.row),
+				sheet_get_view (wbcg->editing_sheet,
+						wb_control_view (WORKBOOK_CONTROL (wbcg))),
 				text, is_array);
 
 			/* Finish editing but do NOT store the results
