@@ -32,14 +32,15 @@
 #include "sheet-object-graphic.h"
 #include "gui-util.h"
 
-#ifdef ENABLE_BONOBO
+#ifdef WITH_BONOBO
 #include <bonobo.h>
 #endif
 #include <libgnome/gnome-i18n.h>
 
 static void
-create_object_command (SheetControlGUI *scg, SheetObject *so)
+create_object_command (WorkbookControlGUI *wbcg, SheetObject *so)
 {
+	SheetControlGUI *scg = wbcg_cur_scg (wbcg);
 	Sheet *sheet = sc_sheet (SHEET_CONTROL (scg));
 
 	scg_mode_create_object (scg, so);
@@ -48,140 +49,125 @@ create_object_command (SheetControlGUI *scg, SheetObject *so)
 }
 
 static void
-cmd_create_label (GtkWidget *widget, WorkbookControlGUI *wbcg)
+cmd_create_label (GtkWidget *ignored, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	Sheet *sheet = sc_sheet (SHEET_CONTROL (scg));
-	SheetObject *so = sheet_widget_label_new (sheet);
-		
-	create_object_command (scg, so);
+	create_object_command (wbcg,
+		g_object_new (sheet_widget_label_get_type(), NULL));
 }
 static void
-cmd_create_frame (GtkWidget *widget, WorkbookControlGUI *wbcg)
+cmd_create_frame (GtkWidget *ignored, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	Sheet *sheet = sc_sheet (SHEET_CONTROL (scg));
-	SheetObject *so = sheet_widget_frame_new (sheet);
-	create_object_command (scg, so);
+	create_object_command (wbcg,
+		g_object_new (sheet_widget_frame_get_type(), NULL));
 }
 
-#if 0 
+#if 0
 	/* not useful until we have scripts */
 static void
 cmd_create_button (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	Sheet *sheet = sc_sheet (SHEET_CONTROL (scg));
-	SheetObject *so = sheet_widget_button_new (sheet);
-	create_object_command (scg, so);
+	create_object_command (wbcg,
+		g_object_new (sheet_widget_button_get_type(), NULL));
 }
 #endif
 static void
+cmd_create_scrollbar (GtkWidget *widget, WorkbookControlGUI *wbcg)
+{
+	create_object_command (wbcg,
+		g_object_new (sheet_widget_scrollbar_get_type(), NULL));
+}
+static void
 cmd_create_checkbox (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	Sheet *sheet = sc_sheet (SHEET_CONTROL (scg));
-	SheetObject *so = sheet_widget_checkbox_new (sheet);
-	create_object_command (scg, so);
+	create_object_command (wbcg,
+		g_object_new (sheet_widget_checkbox_get_type(), NULL));
 }
 #if 0
 static void
 cmd_create_radiobutton (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	Sheet *sheet = sc_sheet (SHEET_CONTROL (scg));
-	SheetObject *so = sheet_widget_radio_button_new	(sheet);
-	create_object_command (scg, so);
+	create_object_command (wbcg,
+		g_object_new (sheet_widget_radiobutton_get_type(), NULL));
 }
 #endif
 static void
 cmd_create_list (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	Sheet *sheet = sc_sheet (SHEET_CONTROL (scg));
-	SheetObject *so = sheet_widget_list_new (sheet);
-	create_object_command (scg, so);
+	create_object_command (wbcg,
+		g_object_new (sheet_widget_list_get_type(), NULL));
 }
 static void
-cmd_create_combobox (GtkWidget *widget, WorkbookControlGUI *wbcg)
+cmd_create_combo (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	Sheet *sheet = sc_sheet (SHEET_CONTROL (scg));
-	SheetObject *so = sheet_widget_combo_new (sheet);
-	create_object_command (scg, so);
+	create_object_command (wbcg,
+		g_object_new (sheet_widget_combo_get_type(), NULL));
 }
 
 static void
 cmd_create_line (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	SheetObject *so = sheet_object_line_new (FALSE);
-	create_object_command (scg, so);
+	create_object_command (wbcg, sheet_object_line_new (FALSE));
 }
 static void
 cmd_create_arrow (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	SheetObject *so = sheet_object_line_new (TRUE);
-	create_object_command (scg, so);
+	create_object_command (wbcg, sheet_object_line_new (TRUE));
 }
 static void
 cmd_create_rectangle (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	SheetObject *so = sheet_object_box_new (FALSE);
-	create_object_command (scg, so);
+	create_object_command (wbcg, sheet_object_box_new (FALSE));
 }
 static void
 cmd_create_ellipse (GtkWidget *widget, WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg = wb_control_gui_cur_sheet (wbcg);
-	SheetObject *so = sheet_object_box_new (TRUE);
-	create_object_command (scg, so);
+	create_object_command (wbcg, sheet_object_box_new (TRUE));
 }
 
-
-#ifndef ENABLE_BONOBO
+#ifndef WITH_BONOBO
 static GnomeUIInfo workbook_object_toolbar [] = {
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("Label"), N_("Creates a label"),
-		&cmd_create_label, "Gnumeric_Label"),
+		&cmd_create_label, "Gnumeric_ObjectLabel"),
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("Frame"), N_("Creates a frame"),
-		&cmd_create_frame, "Gnumeric_Frame"),
+		&cmd_create_frame, "Gnumeric_ObjectFrame"),
 #if 0
 	/* not useful until we have scripts */
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("Button"), N_("Creates a button"),
-		&cmd_create_button, "Gnumeric_Button"),
+		&cmd_create_button, "Gnumeric_ObjectButton"),
 #endif
+	GNOMEUIINFO_ITEM_STOCK (
+		N_("Scrollbar"), N_("Creates a scrollbar"),
+		&cmd_create_scrollbar, "Gnumeric_ObjectScrollbar"),
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("Checkbox"), N_("Creates a checkbox"),
-		&cmd_create_checkbox, "Gnumeric_Checkbutton"),
+		&cmd_create_checkbox, "Gnumeric_ObjectCheckbox"),
 #if 0
 	/* need to think about how to manage groups */
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("RadioButton"), N_("Creates a radio button"),
-		&cmd_create_radiobutton, "Gnumeric_Radiobutton"),
+		&cmd_create_radiobutton, "Gnumeric_ObjectRadiobutton"),
 #endif
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("List"), N_("Creates a list"),
-		&cmd_create_list, "Gnumeric_List"),
+		&cmd_create_list, "Gnumeric_ObjectList"),
 	GNOMEUIINFO_ITEM_STOCK (
-		N_("Combo"), N_("Creates a combobox"),
-		&cmd_create_combobox, "Gnumeric_Combo"),
+		N_("Combo"), N_("Creates a combo box"),
+		&cmd_create_combo, "Gnumeric_ObjectCombo"),
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("Line"), N_("Creates a line object"),
-		cmd_create_line, "Gnumeric_Line"),
+		cmd_create_line, "Gnumeric_ObjectLine"),
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("Arrow"), N_("Creates an arrow object"),
-		cmd_create_arrow, "Gnumeric_Arrow"),
+		cmd_create_arrow, "Gnumeric_ObjectArrow"),
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("Rectangle"), N_("Creates a rectangle object"),
-		cmd_create_rectangle, "Gnumeric_Rectangle"),
+		cmd_create_rectangle, "Gnumeric_ObjectRectangle"),
 	GNOMEUIINFO_ITEM_STOCK (
 		N_("Ellipse"), N_("Creates an ellipse object"),
-		cmd_create_ellipse, "Gnumeric_Oval"),
+		cmd_create_ellipse, "Gnumeric_ObjectEllipse"),
 
 	GNOMEUIINFO_END
 };
@@ -198,10 +184,11 @@ static BonoboUIVerb verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("CreateLabel", cmd_create_label),
 	BONOBO_UI_UNSAFE_VERB ("CreateFrame", cmd_create_frame),
 	/* BONOBO_UI_UNSAFE_VERB ("CreateButton", cmd_create_button), */
+	BONOBO_UI_UNSAFE_VERB ("CreateScrollbar", cmd_create_scrollbar),
 	BONOBO_UI_UNSAFE_VERB ("CreateCheckbox", cmd_create_checkbox),
 	/* BONOBO_UI_UNSAFE_VERB ("CreateRadiobutton", cmd_create_radiobutton), */
 	BONOBO_UI_UNSAFE_VERB ("CreateList", cmd_create_list),
-	BONOBO_UI_UNSAFE_VERB ("CreateCombobox", cmd_create_combobox),
+	BONOBO_UI_UNSAFE_VERB ("CreateCombo", cmd_create_combo),
 	BONOBO_UI_UNSAFE_VERB ("CreateLine", cmd_create_line),
 	BONOBO_UI_UNSAFE_VERB ("CreateArrow", cmd_create_arrow),
 	BONOBO_UI_UNSAFE_VERB ("CreateRectangle", cmd_create_rectangle),
