@@ -40,25 +40,26 @@
 #include "ranges.h"
 #include "file.h"
 #include "str.h"
+#include "gutils.h"
 #include "plugin-util.h"
 #include "gnumeric-gconf.h"
 
+#include <gsf/gsf-libxml.h>
+#include <gsf/gsf-input.h>
+#include <gsf/gsf-input-gzip.h>
+
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <zlib.h>
-#include <string.h>
 #include <gal/util/e-xml-utils.h>
 #include <gal/widgets/e-colors.h>
 #include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-util.h>
+#include <libgnomeprint/gnome-print-config.h>
+
 #include <locale.h>
 #include <math.h>
+#include <errno.h>
+#include <string.h>
 #include <limits.h>
 #ifdef WITH_BONOBO
 #include <bonobo/bonobo-exception.h>
@@ -93,6 +94,7 @@ gnumeric_xml_get_saver (void)
 XmlParseContext *
 xml_parse_ctx_new_full (xmlDocPtr             doc,
 			xmlNsPtr              ns,
+			WorkbookView	     *wb_view,
 			GnumericXMLVersion    version,
 			XmlSheetObjectReadFn  read_fn,
 			XmlSheetObjectWriteFn write_fn,
@@ -110,12 +112,13 @@ xml_parse_ctx_new_full (xmlDocPtr             doc,
 	ctxt->read_fn      = read_fn;
 	ctxt->user_data    = user_data;
 
+	ctxt->wb_view      = wb_view;
+
 	return ctxt;
 }
 
 XmlParseContext *
-xml_parse_ctx_new (xmlDocPtr doc,
-		   xmlNsPtr  ns)
+xml_parse_ctx_new (xmlDocPtr doc, xmlNsPtr  ns, WorkbookView *wb_view)
 {
 	/* HACK : For now we cheat.
 	 * We should always be able to read versions from 1.0.x
@@ -125,7 +128,7 @@ xml_parse_ctx_new (xmlDocPtr doc,
 	 * Freeze the exported version at V9 for now.
 	 */
 	return xml_parse_ctx_new_full (
-		doc, ns, GNUM_XML_V9, NULL, NULL, NULL);
+		doc, ns, wb_view, GNUM_XML_V9, NULL, NULL, NULL);
 }
 
 void
@@ -260,8 +263,7 @@ xml_node_get_cellpos (xmlNodePtr node, char const *name, CellPos *val)
 	buf = xml_node_get_cstr (node, name);
 	if (val == NULL)
 		return FALSE;
-	res = parse_cell_name ((const char *)buf, &val->col, &val->row, TRUE,
-			       &dummy);
+	res = parse_cell_name ((const char *)buf, val, TRUE, &dummy);
 	xmlFree (buf);
 	return res;
 }
@@ -316,6 +318,26 @@ xml_node_set_print_unit (xmlNodePtr node, char const *name,
 	if (tstr) xmlFree (tstr);
 }
 
+static void
+xml_node_set_print_margins (xmlNodePtr node, char const *name,
+			    double points)
+{
+	xmlNodePtr  child;
+	const char *txt = "points";
+	xmlChar       *tstr;
+
+	if (name == NULL)
+		return;
+
+	child = xmlNewChild (node, NULL, (xmlChar const *)name, NULL);
+
+	xml_node_set_points (child, "Points", points);
+
+	tstr = xmlEncodeEntitiesReentrant (node->doc, (xmlChar const *)txt);
+	xml_node_set_cstr (child, "PrefUnit", (const char *)tstr);
+	if (tstr) xmlFree (tstr);
+}
+
 static void
 xml_node_get_print_unit (xmlNodePtr node, PrintUnit * const pu)
 {
@@ -339,6 +361,14 @@ xml_node_get_print_unit (xmlNodePtr node, PrintUnit * const pu)
 	}
 }
 
+static void
+xml_node_get_print_margin (xmlNodePtr node, double *points)
+{
+	g_return_if_fail (node != NULL);
+
+	xml_node_get_double (node, "Points", points);
+}
+
 static gboolean
 xml_node_get_range (xmlNodePtr tree, Range *r)
 {
@@ -369,24 +399,24 @@ static void
 xml_read_selection_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	Range r;
-	int row, col;
-	Sheet *sheet = ctxt->sheet;
+	CellPos pos;
+	SheetView *sv = sheet_get_view (ctxt->sheet, ctxt->wb_view);
 	xmlNodePtr sel, selections = e_xml_get_child_by_name (tree, (xmlChar const *)"Selections");
 
 	if (selections == NULL)
 		return;
 
-	sheet_selection_reset (sheet);
+	sv_selection_reset (sv);
 	for (sel = selections->xmlChildrenNode; sel; sel = sel->next)
 		if (!xmlIsBlankNode (sel) && xml_node_get_range (sel, &r))
-			sheet_selection_add_range (sheet,
-						   r.start.col, r.start.row,
-						   r.start.col, r.start.row,
-						   r.end.col, r.end.row);
+			sv_selection_add_range (sv,
+						r.start.col, r.start.row,
+						r.start.col, r.start.row,
+						r.end.col, r.end.row);
 
-	if (xml_node_get_int (selections, "CursorCol", &col) &&
-	    xml_node_get_int (selections, "CursorRow", &row))
-		sheet_set_edit_pos (sheet, col, row);
+	if (xml_node_get_int (selections, "CursorCol", &pos.col) &&
+	    xml_node_get_int (selections, "CursorRow", &pos.row))
+		sv_set_edit_pos (sv, &pos);
 }
 
 static void
@@ -394,11 +424,13 @@ xml_write_selection_info (XmlParseContext *ctxt, Sheet const *sheet,
 			  xmlNodePtr tree)
 {
 	GList *ptr, *copy;
-	tree = xmlNewChild (tree, ctxt->ns, (xmlChar const *)"Selections",
-			    NULL);
+	SheetView *sv = sheet_get_view (sheet, ctxt->wb_view);
+
+	tree = xmlNewChild (tree, ctxt->ns,
+		(xmlChar const *)"Selections", NULL);
 
 	/* Insert the selections in REVERSE order */
-	copy = g_list_copy (sheet->selections);
+	copy = g_list_copy (sv->selections);
 	ptr = g_list_reverse (copy);
 	for (; ptr != NULL ; ptr = ptr->next) {
 		Range const *r = ptr->data;
@@ -409,8 +441,8 @@ xml_write_selection_info (XmlParseContext *ctxt, Sheet const *sheet,
 	}
 	g_list_free (copy);
 
-	xml_node_set_int (tree, "CursorCol", sheet->edit_pos_real.col);
-	xml_node_set_int (tree, "CursorRow", sheet->edit_pos_real.row);
+	xml_node_set_int (tree, "CursorCol", sv->edit_pos_real.col);
+	xml_node_set_int (tree, "CursorRow", sv->edit_pos_real.row);
 }
 
 /*
@@ -884,24 +916,24 @@ xml_write_attribute (xmlNode *parent, char const *name, char const *value)
 }
 
 static xmlNodePtr
-xml_write_wbv_attributes (XmlParseContext *ctxt, WorkbookView *wbv)
+xml_write_wbv_attributes (XmlParseContext *ctxt)
 {
 	xmlNodePtr attributes = xmlNewDocNode (ctxt->doc, ctxt->ns, "Attributes", NULL);
 	xml_write_attribute (attributes, "WorkbookView::show_horizontal_scrollbar",
-		wbv->show_horizontal_scrollbar ? "TRUE" : "FALSE");
+		ctxt->wb_view->show_horizontal_scrollbar ? "TRUE" : "FALSE");
 	xml_write_attribute (attributes, "WorkbookView::show_vertical_scrollbar",
-		wbv->show_vertical_scrollbar ? "TRUE" : "FALSE");
+		ctxt->wb_view->show_vertical_scrollbar ? "TRUE" : "FALSE");
 	xml_write_attribute (attributes, "WorkbookView::show_notebook_tabs",
-		wbv->show_notebook_tabs ? "TRUE" : "FALSE");
+		ctxt->wb_view->show_notebook_tabs ? "TRUE" : "FALSE");
 	xml_write_attribute (attributes, "WorkbookView::do_auto_completion",
-		wbv->do_auto_completion ? "TRUE" : "FALSE");
+		ctxt->wb_view->do_auto_completion ? "TRUE" : "FALSE");
 	xml_write_attribute (attributes, "WorkbookView::is_protected",
-		wbv->is_protected ? "TRUE" : "FALSE");
+		ctxt->wb_view->is_protected ? "TRUE" : "FALSE");
 	return attributes;
 }
 
 static void
-xml_read_wbv_attributes (XmlParseContext *ctxt, WorkbookView *wbv, xmlNodePtr tree)
+xml_read_wbv_attributes (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNode *attr, *tmp;
 	char *name, *value;
@@ -929,11 +961,11 @@ xml_read_wbv_attributes (XmlParseContext *ctxt, WorkbookView *wbv, xmlNodePtr tr
 			continue;
 		}
 
-		wb_view_set_attribute (wbv, name, value);
+		wb_view_set_attribute (ctxt->wb_view, name, value);
 		xmlFree (name);
 		xmlFree (value);
 	}
-	wb_view_prefs_update (wbv);
+	wb_view_prefs_update (ctxt->wb_view);
 }
 
 static xmlNodePtr
@@ -958,18 +990,20 @@ static xmlNodePtr
 xml_write_print_info (XmlParseContext *ctxt, PrintInformation *pi)
 {
 	xmlNodePtr cur, child;
+	double header = 0, footer = 0, left = 0, right = 0;
 
 	g_return_val_if_fail (pi != NULL, NULL);
 
+	print_info_get_margins (pi, &header, &footer, &left, &right);
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, (xmlChar const *)"PrintInformation", NULL);
 
 	child = xmlNewChild (cur, ctxt->ns, (xmlChar const *)"Margins", NULL);
 	xml_node_set_print_unit (child, "top",    &pi->margins.top);
 	xml_node_set_print_unit (child, "bottom", &pi->margins.bottom);
-	xml_node_set_print_unit (child, "left",   &pi->margins.left);
-	xml_node_set_print_unit (child, "right",  &pi->margins.right);
-	xml_node_set_print_unit (child, "header", &pi->margins.header);
-	xml_node_set_print_unit (child, "footer", &pi->margins.footer);
+	xml_node_set_print_margins (child, "left", left);
+	xml_node_set_print_margins (child, "right", right);
+	xml_node_set_print_margins (child, "header", header);
+	xml_node_set_print_margins (child, "footer", footer);
 
 	child = xmlNewChild (cur, ctxt->ns, (xmlChar const *)"Scale", NULL);
 	if (pi->scaling.type == PERCENTAGE) {
@@ -1013,8 +1047,15 @@ xml_write_print_info (XmlParseContext *ctxt, PrintInformation *pi)
 	xml_node_set_print_hf (cur, "Header", pi->header);
 	xml_node_set_print_hf (cur, "Footer", pi->footer);
 
-	if (pi->paper != NULL)
-		xmlNewChild (cur, ctxt->ns, (xmlChar const *)"paper", (xmlChar const *)pi->paper->name);
+	{
+		gchar *paper_name;
+		paper_name = gnome_print_config_get (pi->print_config, GNOME_PRINT_KEY_PAPER_SIZE);
+		if (paper_name) {
+			xmlNewChild (cur, ctxt->ns, (xmlChar const *)"paper", 
+				     (xmlChar const *)paper_name);
+		}
+		g_free (paper_name);
+	}
 
 	return cur;
 }
@@ -1028,11 +1069,21 @@ xml_write_print_info (XmlParseContext *ctxt, PrintInformation *pi)
 static void
 xml_print_info_fix_margins (PrintInformation *pi)
 {
-	if (pi->margins.top.points < pi->margins.header.points)
-		DSWAP (pi->margins.top.points, pi->margins.header.points);
-	if (pi->margins.bottom.points < pi->margins.footer.points)
-		DSWAP (pi->margins.bottom.points, pi->margins.footer.points);
+	double header = 0, footer = 0, left = 0, right = 0;
+	gboolean switched = FALSE;
 
+	print_info_get_margins (pi, &header, &footer, &left, &right);
+
+	if (pi->margins.top.points < header) {
+		switched = TRUE;
+		DSWAP (pi->margins.top.points, header);
+	}
+	if (pi->margins.bottom.points < footer) {
+		switched = TRUE;
+		DSWAP (pi->margins.bottom.points, footer);
+	}
+	if (switched)
+		print_info_set_margins (pi, header, footer, left, right);
 }
 
 static void
@@ -1040,6 +1091,7 @@ xml_read_print_margins (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr child;
 	PrintInformation *pi;
+	double header = 0, footer = 0, left = 0, right = 0;
 
 	g_return_if_fail (ctxt != NULL);
 	g_return_if_fail (tree != NULL);
@@ -1054,13 +1106,15 @@ xml_read_print_margins (XmlParseContext *ctxt, xmlNodePtr tree)
 	if ((child = e_xml_get_child_by_name (tree, (xmlChar const *)"bottom")))
 		xml_node_get_print_unit (child, &pi->margins.bottom);
 	if ((child = e_xml_get_child_by_name (tree, (xmlChar const *)"left")))
-		xml_node_get_print_unit (child, &pi->margins.left);
+		xml_node_get_print_margin (child, &left);
 	if ((child = e_xml_get_child_by_name (tree, (xmlChar const *)"right")))
-		xml_node_get_print_unit (child, &pi->margins.right);
+		xml_node_get_print_margin (child, &right);
 	if ((child = e_xml_get_child_by_name (tree, (xmlChar const *)"header")))
-		xml_node_get_print_unit (child, &pi->margins.header);
+		xml_node_get_print_margin (child, &header);
 	if ((child = e_xml_get_child_by_name (tree, (xmlChar const *)"footer")))
-		xml_node_get_print_unit (child, &pi->margins.footer);
+		xml_node_get_print_margin (child, &footer);
+
+	print_info_set_margins (pi, header, footer, left, right);
 	xml_print_info_fix_margins (pi);
 }
 
@@ -1196,7 +1250,7 @@ xml_read_print_info (XmlParseContext *ctxt, xmlNodePtr tree)
 
 	if ((child = e_xml_get_child_by_name (tree, (xmlChar const *)"paper"))) {
 		char *name = (char *)xmlNodeGetContent (child);
-		pi->paper = gnome_print_paper_get_by_name (name);
+		gnome_print_config_set (pi->print_config, GNOME_PRINT_KEY_PAPER_SIZE, name);
 		xmlFree (name);
 	}
 }
@@ -1903,6 +1957,7 @@ xml_write_merged_regions (XmlParseContext const *ctxt,
 static void
 xml_read_sheet_layout (XmlParseContext *ctxt, xmlNodePtr tree)
 {
+	SheetView *sv = sheet_get_view (ctxt->sheet, ctxt->wb_view);
 	xmlNodePtr child;
 	CellPos tmp, frozen_tl, unfrozen_tl;
 
@@ -1912,26 +1967,28 @@ xml_read_sheet_layout (XmlParseContext *ctxt, xmlNodePtr tree)
 
 	/* The top left cell in pane[0] */
 	if (xml_node_get_cellpos (tree, "TopLeft", &tmp))
-		sheet_set_initial_top_left (ctxt->sheet, tmp.col, tmp.row);
+		sv_set_initial_top_left (sv, tmp.col, tmp.row);
 
 	child = e_xml_get_child_by_name (tree, (xmlChar const *)"FreezePanes");
 	if (child != NULL &&
 	    xml_node_get_cellpos (child, "FrozenTopLeft", &frozen_tl) &&
 	    xml_node_get_cellpos (child, "UnfrozenTopLeft", &unfrozen_tl))
-		sheet_freeze_panes (ctxt->sheet, &frozen_tl, &unfrozen_tl);
+		sv_freeze_panes (sv, &frozen_tl, &unfrozen_tl);
 }
 
 static void
 xml_write_sheet_layout (XmlParseContext *ctxt, xmlNodePtr tree, Sheet const *sheet)
 {
+	SheetView const *sv = sheet_get_view (sheet, ctxt->wb_view);
+
 	tree = xmlNewChild (tree, ctxt->ns, (xmlChar const *)"SheetLayout", NULL);
 
-	xml_node_set_cellpos (tree, "TopLeft", &sheet->initial_top_left);
-	if (sheet_is_frozen (sheet)) {
+	xml_node_set_cellpos (tree, "TopLeft", &sv->initial_top_left);
+	if (sv_is_frozen (sv)) {
 		xmlNodePtr freeze = xmlNewChild (tree, ctxt->ns, (xmlChar const *)"FreezePanes", NULL);
-		xml_node_set_cellpos (freeze, "FrozenTopLeft", &sheet->frozen_top_left);
+		xml_node_set_cellpos (freeze, "FrozenTopLeft", &sv->frozen_top_left);
 		xml_node_set_cellpos (freeze, "UnfrozenTopLeft",
-				      &sheet->unfrozen_top_left);
+				      &sv->unfrozen_top_left);
 	}
 }
 
@@ -2752,7 +2809,7 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar *buffer, int len
 		return NULL;
 	}
 
-	ctxt = xml_parse_ctx_new (doc, NULL);
+	ctxt = xml_parse_ctx_new (doc, NULL, NULL);
 	cr = cellregion_new (NULL);
 
 	xml_node_get_int (clipboard, "Cols", &cr->cols);
@@ -2819,7 +2876,7 @@ xml_cellregion_write (WorkbookControl *wbc, CellRegion *cr, int *size)
 	g_return_val_if_fail (IS_SHEET (cr->origin_sheet), NULL);
 
 	/* Create the tree */
-	ctxt = xml_parse_ctx_new (xmlNewDoc ((xmlChar const *)"1.0"), NULL);
+	ctxt = xml_parse_ctx_new (xmlNewDoc ((xmlChar const *)"1.0"), NULL, NULL), NULL;
 	clipboard = xmlNewDocNode (ctxt->doc, ctxt->ns, (xmlChar const *)"ClipboardRange", NULL);
 	xml_node_set_int (clipboard, "Cols", cr->cols);
 	xml_node_set_int (clipboard, "Rows", cr->rows);
@@ -2898,6 +2955,9 @@ xml_check_version (xmlDocPtr doc, GnumericXMLVersion *version)
 	xmlNsPtr gmr;
 	int i;
 
+	if (doc == NULL && doc->xmlRootNode == NULL)
+		return NULL;
+
 	/* Do a bit of checking, get the namespaces, and check the top elem.  */
 	if (doc->xmlRootNode->name == NULL || strcmp (doc->xmlRootNode->name, "Workbook"))
 		return NULL;
@@ -2916,13 +2976,13 @@ xml_check_version (xmlDocPtr doc, GnumericXMLVersion *version)
  * Create an XML subtree of doc equivalent to the given Workbook.
  */
 xmlNodePtr
-xml_workbook_write (XmlParseContext *ctxt, WorkbookView *wb_view)
+xml_workbook_write (XmlParseContext *ctxt)
 {
 	xmlNodePtr cur;
 	xmlNodePtr child;
 	GList *sheets, *sheets0;
 	char *old_num_locale, *old_monetary_locale, *old_msg_locale;
-	Workbook *wb = wb_view_workbook (wb_view);
+	Workbook *wb = wb_view_workbook (ctxt->wb_view);
 
 	/*
 	 * General informations about the Sheet.
@@ -2951,7 +3011,7 @@ xml_workbook_write (XmlParseContext *ctxt, WorkbookView *wb_view)
 	old_msg_locale = g_strdup (textdomain (NULL));
 	textdomain ("C");
 
-	child = xml_write_wbv_attributes (ctxt, wb_view);
+	child = xml_write_wbv_attributes (ctxt);
 	if (child)
 		xmlAddChild (cur, child);
 
@@ -2988,8 +3048,8 @@ xml_workbook_write (XmlParseContext *ctxt, WorkbookView *wb_view)
 	xmlAddChild (cur, child);*/
 
 	child = xmlNewDocNode (ctxt->doc, ctxt->ns, (xmlChar const *)"Geometry", NULL);
-	xml_node_set_int (child, "Width", wb_view->preferred_width);
-	xml_node_set_int (child, "Height", wb_view->preferred_height);
+	xml_node_set_int (child, "Width", ctxt->wb_view->preferred_width);
+	xml_node_set_int (child, "Height", ctxt->wb_view->preferred_height);
 	xmlAddChild (cur, child);
 
 	/* sheet content */
@@ -2999,7 +3059,7 @@ xml_workbook_write (XmlParseContext *ctxt, WorkbookView *wb_view)
 	g_list_free (sheets0);
 
 	child = xmlNewDocNode (ctxt->doc, ctxt->ns, (xmlChar const *)"UIData", NULL);
-	xml_node_set_int (child, "SelectedTab", wb_view_cur_sheet (wb_view)->index_in_wb);
+	xml_node_set_int (child, "SelectedTab", wb_view_cur_sheet (ctxt->wb_view)->index_in_wb);
 	xmlAddChild (cur, child);
 
 	textdomain (old_msg_locale);
@@ -3088,13 +3148,13 @@ xml_read_workbook_n_elements (xmlNodePtr tree)
  * Create a Workbook equivalent to the XML subtree of doc.
  */
 gboolean
-xml_workbook_read (IOContext *context, WorkbookView *wb_view,
+xml_workbook_read (IOContext *context,
 		   XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	Sheet *sheet;
 	xmlNodePtr child, c;
 	char *old_num_locale, *old_monetary_locale, *old_msg_locale;
-	Workbook *wb = wb_view_workbook (wb_view);
+	Workbook *wb = wb_view_workbook (ctxt->wb_view);
 
 	if (strcmp (tree->name, "Workbook")){
 		fprintf (stderr,
@@ -3121,7 +3181,7 @@ xml_workbook_read (IOContext *context, WorkbookView *wb_view,
 
 		xml_node_get_int (child, "Width", &width);
 		xml_node_get_int (child, "Height", &height);
-		wb_view_preferred_size	  (wb_view, width, height);
+		wb_view_preferred_size	  (ctxt->wb_view, width, height);
 	}
 
 /*	child = xml_search_child (tree, "Style");
@@ -3166,13 +3226,13 @@ xml_workbook_read (IOContext *context, WorkbookView *wb_view,
 
 	child = e_xml_get_child_by_name (tree, "Attributes");
 	if (child && ctxt->version >= GNUM_XML_V5)
-		xml_read_wbv_attributes (ctxt, wb_view, child);
+		xml_read_wbv_attributes (ctxt, child);
 
 	child = e_xml_get_child_by_name (tree, (xmlChar const *)"UIData");
 	if (child) {
 		int sheet_index = 0;
 		if (xml_node_get_int (child, "SelectedTab", &sheet_index))
-			wb_view_sheet_focus (wb_view,
+			wb_view_sheet_focus (ctxt->wb_view,
 				workbook_sheet_by_index (ctxt->wb, sheet_index));
 	}
 
@@ -3193,7 +3253,7 @@ xml_workbook_read (IOContext *context, WorkbookView *wb_view,
  * passes, then we return TRUE
  */
 static gboolean
-xml_probe (GnumFileOpener const *fo, const gchar *filename, FileProbeLevel pl)
+xml_probe (GnumFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
 {
 	int ret;
 	xmlDocPtr res = NULL;
@@ -3203,7 +3263,7 @@ xml_probe (GnumFileOpener const *fo, const gchar *filename, FileProbeLevel pl)
 	GnumericXMLVersion version;
 
 	if (pl == FILE_PROBE_FILE_NAME) {
-		char const *extension = g_extension_pointer (filename);
+		char const *extension = gnm_extension_pointer (gsf_input_name (input));
 
 		return (extension != NULL &&
 		        (g_strcasecmp (extension, "gnumeric") == 0 ||
@@ -3214,7 +3274,7 @@ xml_probe (GnumFileOpener const *fo, const gchar *filename, FileProbeLevel pl)
 	/*
 	 * Do a silent call to the XML parser.
 	 */
-	ctxt = xmlCreateFileParserCtxt (filename);
+	ctxt = gsf_xml_parser_context (input);
 	if (ctxt == NULL)
 		return FALSE;
 
@@ -3264,90 +3324,71 @@ gnumeric_xml_set_compression (xmlDoc *doc, int compression)
 void
 gnumeric_xml_read_workbook (GnumFileOpener const *fo,
                             IOContext *context,
-                            WorkbookView *wbv,
-                            const gchar *filename)
+                            WorkbookView *wb_view,
+                            GsfInput *input)
 {
-	gint fd;
-	gzFile *f;
-	struct stat sbuf;
-	gint file_size;
-	ErrorInfo *open_error;
-	gchar buffer[XML_INPUT_BUFFER_SIZE];
-	gint bytes;
 	xmlParserCtxtPtr pctxt;
-	xmlDocPtr res;
+	xmlDocPtr res = NULL;
 	xmlNsPtr gmr;
 	XmlParseContext *ctxt;
 	GnumericXMLVersion    version;
+	GsfInputGZip *gzip = NULL;
+	GsfInput *source = NULL;
+	guint8 const *buf;
+	size_t size, len;
 
-	g_return_if_fail (filename != NULL);
+	g_return_if_fail (input != NULL);
 
-	/*
-	 * Load the file into an XML tree.
-	 */
-	fd = gnumeric_open_error_info (filename, O_RDONLY, &open_error);
-	if (fd < 0) {
-		gnumeric_io_error_info_set (context, open_error);
-		return;
-	}
-	if (fstat (fd, &sbuf) == 0) {
-		file_size = sbuf.st_size;
-	} else {
-		gnumeric_io_error_info_set (context, error_info_new_from_errno ());
-		gnumeric_io_error_push (context, error_info_new_str (
-		                        _("Cannot get file size.")));
-		close (fd);
-		return;
-	}
-	f = gzdopen (fd, "r");
-	if (f == NULL) {
-		close (fd);
-		gnumeric_io_error_info_set (context, error_info_new_str (
-		_("Not enough memory to create zlib decompression state.")));
-		return;
-	}
 	io_progress_message (context, _("Reading file..."));
 	io_progress_range_push (context, 0.0, 0.5);
-	value_io_progress_set (context, file_size, 0);
-	bytes = gzread (f, buffer, 4);
-	pctxt = xmlCreatePushParserCtxt (NULL, NULL, buffer, bytes, filename);
-	while ((bytes = gzread (f, buffer, XML_INPUT_BUFFER_SIZE)) > 0) {
-		xmlParseChunk (pctxt, buffer, bytes, 0);
-		value_io_progress_update (context, lseek (fd, 0, SEEK_CUR));
+
+	gzip = gsf_input_gzip_new (input, NULL);
+	source = (gzip != NULL) ? GSF_INPUT (gzip) : input;
+	value_io_progress_set (context, gsf_input_size (source), 0);
+
+	buf = gsf_input_read (source, 4, NULL);
+	size = gsf_input_remaining (source);
+	if (buf != NULL) {
+		pctxt = xmlCreatePushParserCtxt (NULL, NULL,
+			buf, 4, gsf_input_name (source));
+
+		for (; size > 0 ; size -= len) {
+			len = XML_INPUT_BUFFER_SIZE;
+			if (len > size)
+				len =  size;
+		       buf = gsf_input_read (source, len, NULL);
+		       if (buf == NULL)
+			       break;
+		       xmlParseChunk (pctxt, buf, len, 0);
+		       value_io_progress_update (context, gsf_input_tell (source));
+		}
+		xmlParseChunk (pctxt, buf, 0, 1);
+		res = pctxt->myDoc;
+		xmlFreeParserCtxt (pctxt);
 	}
-	xmlParseChunk (pctxt, buffer, 0, 1);
-	res = pctxt->myDoc;
-	xmlFreeParserCtxt (pctxt);
-	gzclose (f);
+
+	if (gzip != NULL)
+		g_object_unref (G_OBJECT (gzip));
 	io_progress_unset (context);
 	io_progress_range_pop (context);
 
-	if (res == NULL) {
-		gnumeric_io_error_read (context, "");
-		return;
-	}
-	if (res->xmlRootNode == NULL) {
-		xmlFreeDoc (res);
-		gnumeric_io_error_read (context,
-			_("Invalid xml file. Tree is empty ?"));
-		return;
-	}
-
 	/* Do a bit of checking, get the namespaces, and check the top elem. */
 	gmr = xml_check_version (res, &version);
 	if (gmr == NULL) {
-		xmlFreeDoc (res);
+		if (res != NULL)
+			xmlFreeDoc (res);
 		gnumeric_io_error_read (context,
 			_("Is not an Gnumeric Workbook file"));
 		return;
 	}
 
 	/* Parse the file */
-	ctxt = xml_parse_ctx_new (res, gmr);
+	ctxt = xml_parse_ctx_new (res, gmr, wb_view);
 	ctxt->version = version;
-	xml_workbook_read (context, wbv, ctxt, res->xmlRootNode);
-	workbook_set_saveinfo (wb_view_workbook (wbv), filename, FILE_FL_AUTO,
-	                       gnumeric_xml_get_saver ());
+	xml_workbook_read (context, ctxt, res->xmlRootNode);
+	workbook_set_saveinfo (wb_view_workbook (ctxt->wb_view),
+		gsf_input_name (input), FILE_FL_AUTO,
+		gnumeric_xml_get_saver ());
 	xml_parse_ctx_destroy (ctxt);
 	xmlFreeDoc (res);
 }
@@ -3375,12 +3416,13 @@ gnumeric_xml_write_workbook (GnumFileSaver const *fs,
 		gnumeric_io_error_save (context, "");
 		return;
 	}
-	ctxt = xml_parse_ctx_new (xml, NULL);
-	xml->xmlRootNode = xml_workbook_write (ctxt, wb_view);
+	ctxt = xml_parse_ctx_new (xml, NULL, wb_view);
+	ctxt->io_context = context;
+	xml->xmlRootNode = xml_workbook_write (ctxt);
 	xml_parse_ctx_destroy (ctxt);
 
 	/* If the suffix is .xml disable compression */
-	extension = g_extension_pointer (filename);
+	extension = gnm_extension_pointer (filename);
 	compression =
 		(extension != NULL && g_strcasecmp (extension, "xml") == 0)
 		? 0 : -1;
